__all__ = ['anciennete_au_club',
           'builds_excel_presence_au_club',
           'count_participation',
           'evolution_age_median',
           'evolution_effectif',
           'inscrit_sejour',
           'plot_rebond',
           'read_effectif',
           'read_effectif_corrected',
           'statistique_vae',
           ]

# Standard library imports
import datetime
import functools
import os
import pathlib
import re
import unicodedata
from collections import Counter
from math import asin, cos, radians, sin, sqrt
from pathlib import Path

# 3rd party imports
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd

# Internal imports
from ctg.ctgfuncts.ctg_tools import built_lat_long
from ctg.ctgfuncts.ctg_tools import read_sortie_csv

def read_effectif_corrected(ctg_path:pathlib.WindowsPath, year=None):

    '''Lecture du fichier effectif et correction
    '''


    if year is not None:
        file_effectif = str(year) + '.xlsx'
    else:
        currentDateTime = datetime.datetime.now()
        date = currentDateTime.date()
        year = date.strftime("%Y")
        file_effectif = year + '.xlsx'

    effectif_df = pd.read_excel(ctg_path / Path(str(year))/Path('DATA')/ Path(file_effectif))
    effectif_df = effectif_df[['N° Licencié', 'Nom','Prénom','Sexe','Pratique VAE']]
    path_root = ctg_path / Path(str(year))/Path('DATA')
    correction_effectif = pd.read_excel(path_root/Path('correction_effectif.xlsx'))
    correction_effectif.index = correction_effectif['N° Licencié']
    root_path = ctg_path / Path(str(year))/Path('DATA')
    membres_sympathisants_df = pd.read_excel(root_path/Path('membres_sympatisants.xlsx'))
    membres_sympathisants_df = membres_sympathisants_df[['N° Licencié',
                                                         'Nom',
                                                         'Prénom',
                                                         'Sexe',
                                                         'Pratique VAE']]

    for num_licence in correction_effectif.index:
        idx = effectif_df[effectif_df["N° Licencié"]==num_licence].index
        effectif_df.loc[idx,'Prénom'] = correction_effectif.loc[num_licence,'Prénom']
        effectif_df.loc[idx,'Nom'] = correction_effectif.loc[num_licence,'Nom']

    effectif_df = pd.concat([effectif_df, membres_sympathisants_df], ignore_index=True, axis=0)
    effectif_df['Prénom1'] = effectif_df['Prénom'].str[0]
    effectif_df['Prénom'] = effectif_df['Prénom'].str.replace(' ','-')

    return effectif_df

def inscrit_sejour(file:pathlib.WindowsPath,no_match:list,df_effectif):

    '''builds the DataFrame dg for one event using the csv file of this event.
    The DataFrame dg has 5 columns named :'N° Licencié','Nom','Prénom','Sexe','sejour'
    And EXCEL file is stored in the corresponding EXCEL directory.
    '''

    # used to supress no ascii characters suh as accent cedilla,...
    nfc = functools.partial(unicodedata.normalize,'NFD')
    convert_to_ascii = lambda text : nfc(text). \
                                     encode('ascii', 'ignore'). \
                                     decode('utf-8').\
                                     strip()

    # read the csv file of the event with full path file
    df = read_sortie_csv(file)

    # extract the event type (SEJOUR, SORTIES DU JEUDI,...)
    sejour = os.path.splitext(os.path.basename(file))[0]

    col = ['N° Licencié','Nom','Prénom','Sexe','Pratique VAE','sejour',]
    if df is not None: # file a valid non-empty file
        dg = df[0].str.upper()
        dg = dg.dropna()
        dg = dg.str.replace(' \t?','',regex=False)
        dg = dg.str.replace('.',' ',regex=False)
        dg = dg.str.replace(' - ','-',regex=False)
        dg = dg.apply(convert_to_ascii)
        dg = dg.drop_duplicates()
        dg = dg.str.split('\s{1,10}')

        dg = dg.apply(lambda row : row+[None] if len(row)==2 else row)

        split_dg = pd.DataFrame(dg.tolist(), columns=['name1', 'name2', 'name3'])

        dic = {}
        for idx,row in split_dg.iterrows():
            if (row.name3 is None) and ( row.name2 is not None):
                if len(row.name1)==1:
                    dr = df_effectif.query('Prénom1==@row.name1[0] and Nom==@row.name2')
                    if len(dr):
                        dic[idx] =dr.iloc[0].tolist()[:-1]+[sejour]
                    else:
                        print(f'no match,{row.name2},{row.name1} dans {sejour}')
                        no_match.append((file,row.name2,row.name1))
                elif len(row.name2)==1:
                    dr = df_effectif.query('Prénom1==@row.name2 and Nom==@row.name1')
                    if len(dr):
                         dic[idx] =dr.iloc[0].tolist()[:-1]+[sejour]
                    else:
                        print(f'no match,{row.name2},{row.name1} dans {sejour}')
                        no_match.append((file,row.name2,row.name1))
                else:
                    if len((dr:=df_effectif.query('Prénom==@row.name2 and Nom==@row.name1'))):
                         dic[idx] =dr.iloc[0].tolist()[:-1]+[sejour]
                    elif len((dr:=df_effectif.query('Prénom==@row.name1 and Nom==@row.name2'))):
                         dic[idx] =dr.iloc[0].tolist()[:-1]+[sejour]
                    else:
                        print((f'{sejour} : no match, '
                               f'prénom/prénom:{row.name2}, '
                               f'nom/prénom: {row.name1}'))
                        no_match.append((file,row.name2,row.name1))
            else:
                print((f'WARNING: incorrect name {row.name1}, '
                       f'{row.name2}, {row.name3} in sejour {sejour}'))

        dg = pd.DataFrame.from_dict(dic).T
        if len(dg) !=0:
            dg.columns = col
        else:
            dg = pd.DataFrame([[None,None,None,None,None,sejour,]], columns=col)

    else:
        dg = pd.DataFrame([[None,None,None,None,None,sejour,]], columns=col)

    return dg


def count_participation(path:pathlib.WindowsPath,
                        ctg_path:pathlib.WindowsPath,
                        year:str,
                        info_rando:pd.DataFrame,
                        ):

    '''Creates the DataFrame df_total with 11 columns:
    'N° Licencié', 'Nom', 'Prénom', 'Sexe', 'Pratique VAE', 'sejour',
       'nbr_jours', 'Type', 'Prénom1', 'sexe', 'VAE'
    'sejour' is set to 'aucun' if the member has participate to no event.
    '''

    flag_sejour = False
    if os.path.split(path)[-1] == 'SEJOUR' :
        flag_sejour = True

    type_sortie_default = os.path.basename(path)
    type_sortie_default = type_sortie_default.split('.', 1)[0]

    df_effectif = read_effectif_corrected(ctg_path,year)

    no_match = []
    df_list = []
    info_sejours = []
    sejours = [x for x in os.listdir( path / Path('CSV')) if x.endswith('.csv')]

    sejours_xlsx = [x for x in os.listdir( path / Path('EXCEL')) if x.endswith('.xlsx')]
    for sejours_xlsx in sejours_xlsx:
        path_file_xlsx = path / Path('EXCEL') / Path(sejours_xlsx)
        os.remove(path_file_xlsx)

    nbr_moyen_participants = 0
    counter = 1
    for sejour in sejours:
        dg = inscrit_sejour( path / Path('CSV') /Path(sejour),no_match,df_effectif)

        if re.findall(r'^\d{4}[-_]',sejour):
            date = sejour[2:10].replace('_','-')
        else:
            date = str(year)[2:4] + '-' +sejour[0:5].replace('_','-')

        dg['nbr_jours'] = 0
        if date in info_rando['date'].tolist():
            dh = info_rando.query('type=="randonnee" and date==@date')
            if not flag_sejour and len(dh) != 0 :
                dg['Type'] = "RANDONNEE"
            else:
                dg['Type'] = type_sortie_default

            dg['nbr_jours'] = 1
            dh = info_rando.query('type=="sejour" and date==@date')
            dg['cout_sejour'] = 0
            if flag_sejour and len(dh) != 0:
                dg['nbr_jours'] = dh['nbr_jours'].tolist()[0]
                dg['cout_sejour'] = 0
                if type_sortie_default == 'SEJOUR':
                    dg['cout_sejour'] = dh['Cout'].tolist()[0]


        if dg.reset_index().loc[0,'Nom'] is not None:
            nbr_inscrits = len(dg)
            if nbr_inscrits != 0:
                nbr_moyen_participants = nbr_moyen_participants +\
                                        (nbr_inscrits - nbr_moyen_participants)/counter
                counter += 1
                long_string = (f'{os.path.split(path)[-1]} :{sejour}, '
                               f"Nombre d'inscrits : {nbr_inscrits}")
                info_sejours.append(long_string)

        df_list.append(dg)

        # Store ae an EXCEL file
        file_store = os.path.splitext(sejour)[0]+'.xlsx'
        dg.to_excel(path / Path('EXCEL') / Path(file_store))

    long_string = ("Nombre d'évènenements : "
                  f"{counter-1}. Nombre moyen de participants : {nbr_moyen_participants}")
    info_sejours.append(long_string)
    info_sejours = '\n'.join(info_sejours)
    info_sejours_path = ctg_path / Path(str(year))
    info_sejours_path = info_sejours_path / Path('STATISTIQUES') / Path('TEXT')
    info_sejours_path = info_sejours_path / Path(type_sortie_default+'.txt')
    with open(info_sejours_path,'w') as f:
        f.write(info_sejours)

    nbr_evenement = counter-1
    if counter-1 > 0 :  #
        df_total = pd.concat(df_list,ignore_index=True)
    else:
        return (None, None, None)


    liste_licence = df_effectif['N° Licencié']
    liste_licence_sejour = df_total['N° Licencié']
    index = list(set(liste_licence)-set(liste_licence_sejour))

    # take care of the member with no participation to the events
    df_non_inscrits = df_effectif.copy()
    df_non_inscrits = df_non_inscrits[df_non_inscrits['N° Licencié'].isin(index)]
    df_non_inscrits['sejour'] = 'aucun'
    df_total = pd.concat([df_total,df_non_inscrits],ignore_index=True)


    return(no_match,df_total,index)


def read_effectif(ctg_path:pathlib.WindowsPath,year:str)->pd.DataFrame:

    def distance_(row)->float:

        phi1, lon1 = dh.query("Ville=='GRENOBLE'")[['long','lat']].values.flatten()
        phi1, lon1 = radians(phi1), radians(lon1)
        phi2, lon2 = radians(row['long']), radians(row['lat'])
        rad = 6371
        dist = 2 * rad * asin(
                                sqrt(
                                    sin((phi2 - phi1) / 2) ** 2
                                    + cos(phi1) * cos(phi2) * sin((lon2 - lon1) / 2) ** 2
                                ))
        return np.round(dist,1)


    df = pd.read_excel(ctg_path / Path(str(year))/ Path('DATA')/Path(str(year)+'.xlsx'))

    df['Date de naissance'] = pd.to_datetime(df['Date de naissance'], format="%d/%m/%Y")

    df['Age']  = df['Date de naissance'].apply(lambda x : (pd.Timestamp(year, 9, 30)-x).days/365)

    dh = built_lat_long(df)

    df['distance'] = df.apply(distance_,axis=1)

    return df

def evolution_effectif(ctg_path:pathlib.WindowsPath):

    # Evolution des effectifs hommes et femme de 2016 à 2022

    def addlabels(x,y,z=None):
        for i in range(len(x)):
            if z is None:
                plt.text(x[i]-0.4,y[i]+5,y[i],size=10)
            else:
                plt.text(x[i]-0.4,y[i]+5,z[i],size=10,rotation='vertical')


    def _evolution_effectif(years):
        nbr_hommes = []
        nbr_femmes = []
        nbr_total = []
        ratio_femmes = []
        for year in years:
            if year == 2006:
                nbr_femmes.append(41)
                nbr_hommes.append(100)
                nbr_total.append(141)
                ratio_femmes.append(str(29)+'%')
            else:
                df_effectif = read_effectif(ctg_path,year)
                nh = len(df_effectif.query('Sexe =="M"'))
                nbr_hommes.append(nh)
                nf = len(df_effectif.query('Sexe =="F"'))
                nbr_femmes.append(nf)
                nbr_total.append(nh + nf)
                ratio_femmes.append(str(int(100*nf/(nh+nf)))+'%')
        return nbr_hommes, nbr_femmes,nbr_total,ratio_femmes

    years = [2006] + [int(x) for x in os.listdir(ctg_path) if re.findall('^\d{4}$',x)]
    nbr_hommes, nbr_femmes,nbr_total,ratio_femmes = _evolution_effectif(years)

    title='Evolution du nombre de membres CTG'

    ax = plt.axes()
    plt.bar(years, nbr_femmes,label= 'Femme')
    addlabels(years, nbr_femmes)
    plt.bar(years, nbr_hommes,bottom=nbr_femmes,label= 'Hommes')
    plt.legend()
    addlabels(years, nbr_total)
    ax.set_xticks(years)
    plt.xticks(rotation=90)
    plt.tight_layout()
    plt.show()

def evolution_age_median(ctg_path:pathlib.WindowsPath):

    def addlabels(x,y,z=None):
        for i in range(len(x)):
            if z is None:
                plt.text(x[i]-0.3,y[i]+0.1,y[i],size=10)
            else:
                plt.text(x[i]-0.2,y[i]+0.1,z[i],size=10,rotation='vertical')

    xt = range(55,90)
    years = [int(x) for x in os.listdir(ctg_path) if re.findall('^\d{4}$',x)]
    nbr_femmes = []
    ratio_femmes = []
    nbr_total = []
    age_mean = []
    age_naturel = []
    for idx,year in enumerate(years):
        file = ctg_path / Path(str(year)) / Path('DATA') / Path(str(year)+'.xlsx')
        df_effectif = pd.read_excel(file)
        df_effectif['Date de naissance'] = pd.to_datetime(df_effectif['Date de naissance'],
                                                                      format="%d/%m/%Y")
        df_effectif['Age']  = df_effectif['Date de naissance'].apply(lambda x :
                                                        (pd.Timestamp(year, 9, 30)-x).days/365)
        age_median = df_effectif['Age'].median()
        age_mean.append(age_median)
        if idx == 0:
           age_median_0 = age_median
           age_naturel.append(age_median_0)
        else:
           age_naturel.append(age_median_0+idx)

    fig, ax = plt.subplots()
    plt.bar(years,age_mean)
    plt.plot(years,age_naturel,"--r")
    plt.ylabel('Age moyen')
    plt.ylim(50,1.1*max(age_naturel))
    addlabels(years,[round(x,1) for x in age_mean])
    linear_model = np.polyfit(years,age_mean,1)
    linear_model_fn = np.poly1d(linear_model)
    plt.xticks(years,years)
    plt.tick_params(axis='x', labelsize=20,rotation=90)
    plt.tick_params(axis='y', labelsize=20)
    plt.ylabel('Age median',size=20)
    x_s =[years[0]-1] + years + [years[-1]+1]
    plt.plot(x_s,linear_model_fn(x_s),"--g")
    plt.title(f'Pente vieillissement : {round(linear_model[0]*12,1)} mois par ans')
    plt.tight_layout()
    plt.show()

def builds_excel_presence_au_club(ctg_path):

    list_date = [int(x) for x in os.listdir(ctg_path) if re.findall('^\d{4}$',x)]
    list_df = []

    for date in list_date:
        df = pd.read_excel(ctg_path /Path(str(date)) / Path('DATA') / Path(str(date)+'.xlsx'))
        df['date'] = date
        list_df.append(df[['N° Licencié','Nom','Prénom','date']])

    df = pd.concat(list_df) #.to_excel(file / Path('effectif_total.xlsx'),index=False)

    dic = {}
    list_num_licence = []
    list_nom = []
    list_prenom = []
    list_date_ = []
    for licence in df.groupby('N° Licencié'):
        list_c = [x if x in licence[1]['date'].to_list() else None for x in list_date]
        #if len(list_c) - list_c.count(None) == 1: singleton
        list_num_licence.append(licence[0])
        list_nom.append(licence[1]['Nom'].unique()[0])
        list_prenom.append(licence[1]['Prénom'].unique()[0])
        list_date_.append(list_c)


    dic['N° Licencié'] = list_num_licence
    dic['Nom'] = list_nom
    dic['Prénom'] = list_prenom
    dic['date'] = list_date_

    df = pd.DataFrame.from_dict(dic)
    split_df = pd.DataFrame(df['date'].tolist(), columns=list_date)

    df = pd.concat([df, split_df], axis=1)
    df = df.drop('date',axis=1)
    out_path = ctg_path / Path(str(list_date[-1]))
    out_path = out_path / Path('STATISTIQUES') / Path('EXCEL') / Path('effectif_history.xlsx')
    df.to_excel(out_path)
    return out_path

def statistique_vae(ctg_path):

    # function to add value labels
    def addlabels(x,y):
        for i in range(1,len(x)):
            plt.text(x[i]-0.2,y[i]+0.5,y[i],size=10)

    current_year = datetime.datetime.now().year

    last_year = 2018
    years =[]
    nb_vae_m = []
    nb_vae_f = []
    nb_vae_tot = []
    for year in range(last_year,current_year+1):
        df_N1 = pd.read_excel(ctg_path / Path(str(year)) / Path('DATA') /Path(str(year)+'.xlsx'))
        years.append(year)
        nb_vae_m.append(sum((df_N1['Pratique VAE'] == 'Oui') & (df_N1['Sexe'] == 'M')))
        nb_vae_f.append(sum((df_N1['Pratique VAE'] == 'Oui') & (df_N1['Sexe'] == 'F')))
        nb_vae_tot.append(sum((df_N1['Pratique VAE'] == 'Oui') & (df_N1['Sexe'] == 'F')) +
                          sum((df_N1['Pratique VAE'] == 'Oui') & (df_N1['Sexe'] == 'M')))
    plt.bar(years, nb_vae_m,label= 'Homme')
    plt.bar(years, nb_vae_f,bottom=nb_vae_m,label= 'Femme')
    plt.tick_params(axis='x', labelsize=15)
    plt.tick_params(axis='y', labelsize=15)
    plt.ylabel('nombre de VAEs',size=15)
    plt.xticks(rotation=90)
    plt.legend()
    plt.ylabel('nombre de VAE')
    addlabels(years, nb_vae_m)
    addlabels(years, [x+y for x,y in zip(nb_vae_m,nb_vae_f)])
    plt.tight_layout()
    plt.show()

def anciennete_au_club(ctg_path):

    def addlabels(x,y,offset):
        for i in range(len(x)):
            if y[i] != 0:
                plt.text(x[i]-0.2,y[i]+offset,round(y[i],1),size=15)

    currentDateTime = datetime.datetime.now()
    date = currentDateTime.date()
    current_year = int(date.strftime("%Y"))

    in_path = ctg_path / Path(str(current_year))
    in_path = in_path / Path('STATISTIQUES') /Path('EXCEL') / Path('effectif_history.xlsx')
    df = pd.read_excel(in_path)
    eff = []

    years = list(range(2012,current_year+1))
    for year in years:
        dg = df.dropna(subset=[year,current_year])
        eff.append(len(dg))
    eff = [eff[0]] + list(np.diff(eff))

    # creating the bar plot
    fig = plt.figure(figsize = (10, 5))
    plt.bar(years, eff, color ='maroon',
            width = 0.4)

    plt.xlabel("")
    plt.ylabel("# adhérents")
    plt.title("Ancienneté au CTG")
    addlabels(list(range(2012,2025)), eff,0)
    plt.tight_layout()
    plt.show()


def plot_rebond(ctg_path:pathlib.WindowsPath):

    '''
    '''
    
    def addlabels(x,y):
        for i in range(len(x)):
            ax[0].text(i-0.2,-23,y[i],size=10)


    current_year = int(datetime.datetime.now().year)
    file_path = ctg_path / Path(str(current_year)) / Path('STATISTIQUES') / Path('EXCEL')
    file_path = file_path / Path('effectif_history.xlsx')
    if not os.path.isfile(file_path):
        builds_excel_presence_au_club(ctg_path)

    years_list = [int(x) for x in os.listdir(ctg_path) if re.findall('^\d{4}$',x)]
    year_dep = min(years_list)+2
        
    df = pd.read_excel(file_path)
    df = df.fillna(0)
    dic = {}
    years = range(year_dep,current_year+1)
    for year in years:
        list_rebond = []
        list_entrant = []
        list_sortant = []
        for idx,row in df.iterrows():
            if (row[year-2] == 0 and row[year-1] == year-1 and row[year] ==0):
                #print(year,row['Nom'],row['Prénom'],row['N° Licencié'])
                list_rebond.append('-'.join([row['Nom'],row['Prénom']]))
            if row[year-1] == 0 and row[year] == year:
                list_entrant.append('-'.join([row['Nom'],row['Prénom']]))
            if row[year] == 0 and row[year-1] == year-1:
                list_sortant.append('-'.join([row['Nom'],row['Prénom']]))
        dic[year]=[len(list_rebond),'; '.join(list_rebond),
                   len(list_entrant),'; '.join(list_entrant),
                   -len(list_sortant),'; '.join(list_sortant)]
        
    dg = pd.DataFrame.from_dict(dic).T
    dg.columns=['# rebonds',
                'Nom rebonds',
                '# entrants',
                'Nom entrants',
                '# sortants',
                'Nom sortants']
    rebond_pourcent = [None] + [round(100*x[0]/x[1],1) for x in 
                       zip(dg['# rebonds'].tolist()[1:], dg['# entrants'].tolist()[:-1])]
    dg['% rebond'] = rebond_pourcent
    file_rebond = ctg_path / Path(str(current_year)) / Path('STATISTIQUES')
    file_rebond = file_rebond / Path('EXCEL') / Path('rebond.xlsx')
    dg.to_excel(file_rebond)
    year_pourcent = list(years[1:])
    
    fig, ax = plt.subplots(nrows=1, ncols=2)
    dg[['# entrants','# sortants']].plot.bar(stacked=True,ax=ax[0])
    addlabels(dg.index.tolist(),(dg['# entrants']+dg['# sortants']).tolist())
    ax[0].yaxis.grid()
    ax[0].set_ylabel('# membres')
    ax[0].set_ylim((-25,40))
    ax[0].legend(loc='upper center',ncol=1)
    
    dg[['# rebonds']].plot.bar(ax=ax[1])
    ax[1].yaxis.grid()
    plt.show()


__all_ = ["built_lat_long",
          "plot_ctg",
          "stat_sorties_club",]

# Standard library import
import datetime
import operator
import os
from collections import Counter
from math import asin, cos, radians, sin, sqrt
from pathlib import Path
from tkinter import messagebox

# 3rd party imports
import folium
import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import pandas

# Internal imports
from ctg.ctgfuncts.ctg_effectif import read_effectif
from ctg.ctgfuncts.ctg_effectif import read_effectif_corrected
from ctg.ctgfuncts.ctg_effectif import count_participation
from ctg.ctgfuncts.ctg_tools import parse_date
from ctg.ctgfuncts.ctg_tools import built_lat_long

def plot_ctg(ctg_path,year:str):

    '''generates an html file of the membrers geographical location using the
    column ville of the DataFrame df
    '''

    trace_radius = True
    file = ctg_path / Path(f'{str(year)}/DATA/{str(year)}.xlsx')
    df = pd.read_excel(file)
    
    trace_radius = True
    dh = built_lat_long(df)

    villes_set = set(dh['Ville'])
    dh = dh.dropna()
    villes1_set = set(dh['Ville'])
    if len(villes_set-villes1_set) !=0:
        messagebox.showwarning('Villes non reconnues',';'.join(list(villes_set-villes1_set)))

    group_adjacent = lambda a, k: list(zip(*([iter(a)] * k)))

    dict_cyclo = {}
    for ville,y in df.groupby(['Ville'])['Nom']:
        chunk = []
        for i in range(0,len(y),3):
            chunk.append(','.join(y[i:i+3] ))

        dict_cyclo[ville] = '\n'.join(chunk)
    dict_cyclo = {k[0]:v for k,v in dict_cyclo.items()}
    kol = folium.Map(location=[45.2,5.7], tiles='openstreetmap', zoom_start=12)

    long_genoble, lat_grenoble = dh.query("Ville=='GRENOBLE'")[['long','lat']].values.flatten()
    if trace_radius:
        folium.Circle(
                      location=[lat_grenoble, long_genoble],
                      radius=8466,
                      popup='50 km ',
                      color="black",
                      fill=False,
                      ).add_to(kol)
                      
    for latitude,longitude,size, ville in zip(dh['lat'],dh['long'],dh['number'],dh['Ville']):

        long_ville, lat_ville =dh.query("Ville==@ville")[['long','lat']].values.flatten()
        dist_grenoble_ville = _distance(lat_grenoble, long_genoble,lat_ville, long_ville )
        color='red' if dist_grenoble_ville>19.35 else 'blue'
        if ville == "grenoble":
            folium.Circle(
                location=[latitude, longitude],
                radius=size*50,
                popup=f'{ville} ({size}): {dict_cyclo[ville]} ',
                color="yellow",
                fill=True,
            ).add_to(kol)
        else:
                folium.Circle(
                location=[latitude, longitude],
                radius=size*100,
                popup=f'{ville} ({size}): {dict_cyclo[ville]}',
                color=color,
                fill=True,
            ).add_to(kol)
    dict_cyclo_l = {k:len(v.split(',')) for k,v in dict_cyclo.items()}
    list_villes = sorted(dict_cyclo_l.items(), key=operator.itemgetter(1),reverse=True)
    list_villes = '\n\n'.join([f'{t[0]} ({str(t[1])}) : {dict_cyclo[t[0]]}' for t in list_villes])

    file = ctg_path / Path(f'{str(year)}/STATISTIQUES/TEXT/info_effectif_{str(year)}.txt')
    with open(file,'a',encoding='utf-8') as f:
        f.write(f'\n\nNombre de villes : {len(dict_cyclo)}\n')
        f.write(list_villes)
        
    return kol

def stat_sorties_club(path_sorties_club, ctg_path, ylim=None, file_label=None,year = None):

    ''' to do
    '''

    def addlabels(x,y):

        for i in range(len(x)):
            d = x[i]
            color = 'g'
            if os.path.split(path_sorties_club)[-1] == "SEJOUR" :
                v = info_rando.query('date==@d and type=="sejour"')['name_activite'].tolist()
            else:
                v = info_rando.query('date==@d and type!="sejour"')['name_activite'].tolist()
                t = info_rando.query('date==@d and type!="sejour"')['type'].tolist()
                if len(t) != 0:
                    color = "k" if t[0] =='randonnee' else "g"

            name = v[0] if len(v) != 0 else ""

            plt.text(i-0.2,y[i]+1,
                     name,
                     size=10,
                     rotation=90,
                     color=color #info_rando['color']
                     )

    if file_label is not None and os.path.isfile(file_label):
        flag_labels = True
        info_rando = pd.read_excel(file_label)
    else:
        flag_labels = False
        print(file_label)

    no_match,df_total,_ = count_participation(path_sorties_club,ctg_path,year,info_rando,)
    if no_match is None:
        messagebox.showinfo('WARNING',"Aucun participant n'a participé à ce type de sortie" )
    else:
        text_message = ''
        for tup in no_match:
              text_message += (f'Le nom {tup[1]}, {tup[2]} '
                               f'est inconnu dans le fichier : {os.path.split(tup[0])[-1]}')
              text_message += '\n'

        if len(text_message) : messagebox.showinfo('WARNING',text_message )

    if year is None:
        currentDateTime = datetime.datetime.now()
        date = currentDateTime.date()
        year = date.strftime("%Y")

    df_effectif = read_effectif_corrected(ctg_path,year)
    df_total = df_total[df_total['sejour']!='aucun' ] # skip the member with no event
    df_total['sejour'] = df_total['sejour'].apply(lambda s:
                                           parse_date(s,str(year)).strftime('%y-%m-%d'))
    df_total = df_total.fillna(0)

    dic_sexe = dict(M="Homme",F="Femme")
    dic_vae = dict(Oui="VAE",Non="Musculaire")
    df_total = df_total.replace({"Sexe": dic_sexe})
    df_total = df_total.replace({"Pratique VAE": dic_vae})
    if df_total['Nom'].isna().all():
        return None
    dg = df_total.groupby(['Sexe','Pratique VAE'])['sejour'].value_counts().unstack().T

    fig, ax = plt.subplots(figsize=(15, 5))

    dg[['Femme','Homme']].plot(kind='bar',
                       ax=ax,
                       width=0.5,
                       stacked=True,
                       color = {('Femme', 'Musculaire'): '#1f77b4',
                                ('Femme', 'VAE'): '#ff7f0e',
                                ('Homme', 'Musculaire'): '#2ca02c',
                                ('Homme', 'VAE'): '#d62728',} )

    if flag_labels : addlabels(dg.index,dg.sum(axis=1).astype(int).tolist())

    plt.xlabel('')
    plt.tick_params(axis='x', rotation=90,labelsize=15)
    plt.ylabel('Nombre de licenciers',size=15)
    plt.xlabel('')
    plt.tick_params(axis='x', rotation=90,labelsize=15)
    plt.tick_params(axis='y',labelsize=15)
    type_sortie = os.path.split(path_sorties_club)[-1] + ' ' + str(year)
    plt.title(type_sortie,fontsize=15,pad=50)

    if ylim is not None:
        plt.ylim(ylim)
    else:
        ylim = (0,1.5*max(Counter(df_total['sejour']).values()))
        plt.ylim(ylim)


    plt.legend(bbox_to_anchor =(0.75, 1.15), ncol = 2)
    plt.tight_layout()
    plt.show()
    fig_file = os.path.split(path_sorties_club)[-1].replace(' ','_')+'.png'
    file = ctg_path / Path(str(year)) / Path('STATISTIQUES/IMAGE') / Path(fig_file)
    plt.savefig(file,bbox_inches='tight')

    return df_total

def _distance(ϕ1:float, λ1:float,ϕ2:float, λ2:float)->float:

    '''Computes the distance in kilometers between to points referenced
    by there longitudes (in decimal degrees) and there latitudes (in decimal degrees)
    '''

    ϕ1, λ1 = radians(ϕ1), radians(λ1)
    ϕ2, λ2 = radians(ϕ2), radians(λ2)
    rad = 6371 # Earth radius [km]
    dist = 2 * rad * asin(
                          sqrt(
                               sin((ϕ2 - ϕ1) / 2) ** 2
                             + cos(ϕ1) * cos(ϕ2) * sin((λ2 - λ1) / 2) ** 2
                            ))
    return dist


__all__ = ['ACTIVITE_LIST',
           'ADD_SPACE_MM',
           'BM_GUI_DISP',
           'BUTT_AGE_ANALYSIS',
           'BUTT_EFFECTIF_ANALYSIS',
           'BUTT_EVOLUTION_EFFECTIF',
           'BUTT_GEO',
           'BUTT_MEMBER_ANALYSIS',
           'BUTT_NBR_SEJOURS',
           'BUTT_PRESENCE_EFFECTIF',
           'BUTT_RANDO',
           'BUTT_REBOND',
           'BUTT_SEJOUR',
           'BUTT_SORTIES',
           'BUTT_SYNTHESE_SORTIES',
           'BUTT_TENDANCE_SORTIES',
           'CONTAINER_BUTTON_HEIGHT_PX',
           'DIR_SORTIES_LIST',
           'FIRST_YEAR',
           'FONT_NAME',
           'HELP_EVOLUTION_EFFECTIF',
           'HELP_GEO',
           'HELP_MEMBER_ANALYSIS',
           'HELP_NBR_SEJOURS',
           'HELP_PRESENCE_EFFECTIF',
           'HELP_RANDO',
           'HELP_SEJOUR',
           'HELP_SORTIES',
           'HELP_SYNTHESE_SORTIES',
           'HELP_TENDANCE_SORTIES',
           'IN_TO_MM',
           'PAGES_LABELS',
           'PPI',
           'REF_BMF_FONT_SIZE',
           'REF_BMF_POS_X_MM',
           'REF_BMF_POS_Y_MM',
           'REF_BUTTON_DX_MM',
           'REF_BUTTON_DY_MM',
           'REF_BUTTON_FONT_SIZE',
           'REF_CHECK_BOXES_SEP_SPACE',
           'REF_COPYRIGHT_FONT_SIZE',
           'REF_COPYRIGHT_X_MM',
           'REF_COPYRIGHT_Y_MM',
           'REF_CORPI_POS_X_MM',
           'REF_CORPI_POS_Y_MM',
           'REF_ENTRY_NB_CHAR',
           'REF_ETAPE_BUT_DX_MM',
           'REF_ETAPE_BUT_DY_MM',
           'REF_ETAPE_CHECK_DY_MM',
           'REF_ETAPE_FONT_SIZE',
           'REF_ETAPE_POS_X_MM',
           'REF_ETAPE_POS_Y_MM_LIST',
           'REF_EXIT_BUT_POS_X_MM',
           'REF_EXIT_BUT_POS_Y_MM',
           'REF_INST_POS_X_MM',
           'REF_INST_POS_Y_MM',
           'REF_LABEL_FONT_SIZE',
           'REF_LABEL_POS_Y_MM',
           'REF_LAUNCH_FONT_SIZE',
           'REF_PAGE_TITLE_FONT_SIZE',
           'REF_PAGE_TITLE_POS_Y_MM',
           'REF_SCREEN_HEIGHT_MM',
           'REF_SCREEN_HEIGHT_PX',
           'REF_SCREEN_WIDTH_MM',
           'REF_SCREEN_WIDTH_PX',
           'REF_SORTIES_BUT_POS_X_MM',
           'REF_SORTIES_BUT_POS_Y_MM',
           'REF_SUB_TITLE_FONT_SIZE',
           'REF_VERSION_FONT_SIZE',
           'REF_VERSION_X_MM',
           'REF_WINDOW_HEIGHT_MM',
           'REF_WINDOW_WIDTH_MM',
           'REF_YEAR_BUT_POS_X_MM',
           'REF_YEAR_BUT_POS_Y_MM',
           'TEXT_ACTIVITE_PI',
           'TEXT_BOUTON_CREATION_CORPUS',
           'TEXT_BOUTON_LANCEMENT',
           'TEXT_COPYRIGHT',
           'TEXT_CORPUSES',
           'TEXT_EVOLUTION_EFFECTIF',
           'TEXT_GEO',
           'TEXT_GEO_PI',
           'TEXT_INSTITUTE',
           'TEXT_LAUNCH_PARSING',
           'TEXT_LAUNCH_SYNTHESE',
           'TEXT_MEMBER_ANALYSIS',
           'TEXT_METHOD',
           'TEXT_NBR_SEJOURS',
           'TEXT_PAUSE',
           'TEXT_PRESENCE_EFFECTIF',
           'TEXT_RANDO',
           'TEXT_ROOT_CTG',
           'TEXT_ROOT_CTG_CHANGE',
           'TEXT_SEJOUR',
           'TEXT_SORTIES',
           'TEXT_SORTIES_PI',
           'TEXT_SYNTHESE_SORTIES',
           'TEXT_TENDANCE_SORTIES',
           'TEXT_TITLE',
           'TEXT_UPDATE_STATUS',
           'TEXT_VERSION',
           'TEXT_YEAR_PC',
           'TEXT_YEAR_PI',]
           
# Standard HELP_SEJOURlibrary imports
import math           

# 3rd party imports
from screeninfo import get_monitors

################################## General globals ##################################

# Setting BiblioMeter version value (internal)
VERSION ='5.0.0'

# Setting the first available year -1 for activity following
FIRST_YEAR = 2021

# Setting the title of the application main window (internal)
APPLICATION_WINDOW_TITLE = f"CTG_Meter - Analyse des statistiques des effectifs et des sorties"

######################## Definition of display globals ###########################

def _get_displays(in_to_mm):

    ''' The function `get_displays` allows to identify the set of displays
        available within the user hardware and to get their parameters.
        If the width or the height of a display are not available in mm
        through the `get_monitors` method (as for Darwin platforms),
        the user is asked to specify the displays diagonal size to compute them.

    Returns:
        `list`: list of dicts with one dict per detected display,
                each dict is keyed by 8 display parameters.
    '''
    # To Do: convert prints and inputs to gui displays and inputs

    displays = [{'x':m.x,'y':m.y,'width':m.width,
                 'height':m.height,'width_mm':m.width_mm,
                 'height_mm':m.height_mm,'name':m.name,
                 'is_primary':m.is_primary} for m in get_monitors()]

    for disp in range(len(displays)):
        width_px = displays[disp]['width']
        height_px = displays[disp]['height']
        diag_px = math.sqrt(int(width_px)**2 + int(height_px)**2)
        width_mm = displays[disp]['width_mm']
        height_mm = displays[disp]['height_mm']
        if width_mm is None or height_mm is None:
            diag_in = float(input(('Enter the diagonal size of the screen n°'
                                   f'{str(disp)} (inches)')))
            width_mm = round(int(width_px) * (diag_in/diag_px) * in_to_mm,1)
            height_mm = round(int(height_px) * (diag_in/diag_px) * in_to_mm,1)
            displays[disp]['width_mm'] = str(width_mm)
            displays[disp]['height_mm'] = str(height_mm)
        else:
            diag_in = math.sqrt(float(width_mm) ** 2 + float(height_mm) ** 2) / in_to_mm
        displays[disp]['ppi'] = round(diag_px/diag_in,2)

    return displays

# Conversion factor for inch to millimeter
IN_TO_MM = 25.4

DISPLAYS = _get_displays(IN_TO_MM)

# Setting primary display
BM_GUI_DISP = 0

# Getting display resolution in pixels per inch
PPI = DISPLAYS[BM_GUI_DISP]['ppi']

# Setting display reference sizes in pixels and mm (internal)
REF_SCREEN_WIDTH_PX       = 1920
REF_SCREEN_HEIGHT_PX      = 1080
REF_SCREEN_WIDTH_MM       = 467
REF_SCREEN_HEIGHT_MM      = 267

# Application window reference sizes in mm for the display reference sizes (internal)
REF_WINDOW_WIDTH_MM       = 219
REF_WINDOW_HEIGHT_MM      = 173


####################################################################
##########################  Pages globals ##########################
####################################################################


# Setting general globals for text edition
FONT_NAME = "Helvetica"

########################## Reference coordinates for pages ##########################

# Number of characters reference for editing the entered files-folder path
REF_ENTRY_NB_CHAR          = 100     #100

# Font size references for page label and button
REF_SUB_TITLE_FONT_SIZE    = 15      #15
REF_PAGE_TITLE_FONT_SIZE   = 30      #30
REF_LAUNCH_FONT_SIZE       = 25      #25
REF_BMF_FONT_SIZE          = 15      #15        #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
REF_BUTTON_FONT_SIZE       = 12      #10
REF_COPYRIGHT_FONT_SIZE    = 12      #10
REF_VERSION_FONT_SIZE      = 12      #10

# Y position reference in mm for page label
REF_PAGE_TITLE_POS_Y_MM    = 20      #20

# Positions reference in mm for institute selection button
REF_INST_POS_X_MM          = 5       #5
REF_INST_POS_Y_MM          = 40      #45

# Positions reference in mm for bmf label and button
REF_BMF_POS_X_MM           = 5       #5
REF_BMF_POS_Y_MM           = 55      #45
REF_BUTTON_DX_MM           = -147    #-147
REF_BUTTON_DY_MM           = 10      #10

# Positions reference in mm for corpus creation button
REF_CORPI_POS_X_MM         = 5       #5
REF_CORPI_POS_Y_MM         = 85      #75

# Space between label and value
ADD_SPACE_MM               = 10      #10

# Setting X and Y positions reference in mm for copyright
REF_COPYRIGHT_X_MM         = 5       #5
REF_COPYRIGHT_Y_MM         = 170     #170
REF_VERSION_X_MM           = 185     #185

# Container button height in pixels
CONTAINER_BUTTON_HEIGHT_PX = 50      #50

# Font size references for page label and button
REF_LABEL_FONT_SIZE        = 25      #25
REF_ETAPE_FONT_SIZE        = 14      #14
REF_BUTTON_FONT_SIZE       = 10      #10

# Positions reference in mm for pages widgets
REF_LABEL_POS_Y_MM         = 15      #15
REF_ETAPE_POS_X_MM         = 10      #10
REF_ETAPE_POS_Y_MM_LIST    = [40, 74, 101, 129]   #[40, 74, 101, 129]
REF_ETAPE_BUT_DX_MM        = 10      #10
REF_ETAPE_BUT_DY_MM        = 5       #5
REF_ETAPE_CHECK_DY_MM      = -8      #-8
REF_EXIT_BUT_POS_X_MM      = 193     #193
REF_EXIT_BUT_POS_Y_MM      = 145     #145
REF_YEAR_BUT_POS_X_MM      = 10      #10
REF_YEAR_BUT_POS_Y_MM      = 26      #26
REF_SORTIES_BUT_POS_X_MM   = 10      #10
REF_SORTIES_BUT_POS_Y_MM   = 50      #26

# Separation space in mm for check boxes
REF_CHECK_BOXES_SEP_SPACE  = 25      #25

# Setting label for each gui page
PAGES_LABELS = {'page_effectif': "Analyse des effectifs",
                'pagesorties' : "Analyse des sorties",
                'pagesynthese': "Synthèse des sorties et effectifs",
                'pagetendance': "Analyse de l'évolution temporelle des pratiques ",
                'pagetendance': "Analyse tendancielle",
                'pagedivers'  : "Autres analyses",
                'PageHelp'    : "Aide",
               }


########################## Cover Page (BiblioMeter launching Page) ##########################

# Titre de la page
TEXT_TITLE                  = "- CTG_Meter -\nInitialisation de l'analyse"

# Choix de l'année de l'Institut
TEXT_INSTITUTE              = "Sélection de l'Institut"

# Titre LabelEntry of BiblioMeter_Files folder
TEXT_ROOT_CTG                    = "Dossier de travail "

# Titre bouton changement de dossier de travail
TEXT_ROOT_CTG_CHANGE             = "Changer de dossier de travail"

# Titre liste des corpus analysés
TEXT_CORPUSES               = "Liste des années "

# Titre bouton création d'un dossier nouveau de corpus
TEXT_BOUTON_CREATION_CORPUS = "Créer une nouvelle année"

# Titre bouton de lancement
TEXT_BOUTON_LANCEMENT       = "Lancer l'analyse"

# Copyright and contacts
TEXT_COPYRIGHT              =   "Contributeurs et contacts :"
TEXT_COPYRIGHT             +=  "\n- Amal Chabli"
TEXT_COPYRIGHT             +=  "\n- François Bertin : francois.bertin7@wanadoo.fr"
TEXT_COPYRIGHT             +=  "\n- Ludovic Desmeuzes"
TEXT_VERSION                = f"\nVersion {VERSION}"

##########################  Secondary pages ##########################

# Common to secondary pages
TEXT_PAUSE           = "Mettre en pause"

###############  Parsing page

# - Label ANNEE
TEXT_YEAR_PC         = "Sélection de l'année "

# - Bouton mise à jour du statut des fichiers
TEXT_UPDATE_STATUS   = "Mettre à jour le statut des fichiers"

# - Bouton lancement parsing
TEXT_LAUNCH_PARSING  = "Lancer le Parsing"

# - Bouton lancement concatenation et deduplication des parsings
TEXT_LAUNCH_SYNTHESE = "Lancer la synthèse"


###############  Consolidation page

# Choix de l'année de travail
TEXT_YEAR_PI       = "Sélection de l'année "

TEXT_SORTIES_PI       = "Sélection du type de sortie "


###############  Analysis page

### - Page effectif
TEXT_EFFECTIF     = "Analyse de l'effectif"
HELP_EFFECTIF     = " L'analyse des effectifs est effectuée à partir des fichiers"
HELP_EFFECTIF    += " EXCEL ffcyclo.org."
BUTT_EFFECTIF_ANALYSIS = "Lancer l'analyse des effectifs"
TEXT_EVOLUTION_EFFECTIF     = "Analyse de l'évolution temporelle de l'effectif"
HELP_EVOLUTION_EFFECTIF     = (" L'analyse de l'évolution de l'effectif est issue"
                               " des archives disponnibles à partir de 2012.")
BUTT_EVOLUTION_EFFECTIF = "Lancer l'analyse de l'évolution des effectifs"
BUTT_REBOND = "Lancer l'analyse de l'évolution du rebond"
TEXT_AGE_ANALYSIS     = "Analyse de l'évolution de l'âge médian"
HELP_AGE_ANALYSIS     = " L'analyse de l'évolution de l'âge médian"
HELP_AGE_ANALYSIS    += " des archives disponnibles à partir de 2012."
BUTT_AGE_ANALYSIS = "Lancer l'analyse de l'évolution de l'âge médian"

### - sorties
TEXT_SORTIES     = "Analyse des sorties"
HELP_SORTIES     = " L'analyse des sorties est effectuée à partir des fichiers"
HELP_SORTIES    += " csv issues des Framadate"
BUTT_SORTIES = "Lancer l'analyse des sorties"
TEXT_RANDO       = "Analyse des randonnées/séjours"
HELP_RANDO       = " L'analyse des sorties/séjours est effectuée à partir des fichers"
HELP_RANDO      += " d'émargements des participants aux randonnées/séjours"
BUTT_RANDO = "Lancer l'analyse"

### - Page synthese
TEXT_SYNTHESE_SORTIES     = "Synthèse des sorties"
HELP_SYNTHESE_SORTIES     = " La synthèse est effectuée à partir des fichiers"
HELP_SYNTHESE_SORTIES    += " d'émargement aux sorties, randonnées et séjours"
BUTT_SYNTHESE_SORTIES = "Lancer la synthèse"
TEXT_EVOLUTION_EFFECTIF     = "Analyse de l'évolution temporelle de l'effectif"
HELP_EVOLUTION_EFFECTIF     = " L'analyse de l'évolution de l'effectif est issue"
HELP_EVOLUTION_EFFECTIF    += " des archives disponnibles à partir de 2012."
BUTT_EVOLUTION_EFFECTIF = "Lancer l'analyse de l'évolution des effectifs"
TEXT_MEMBER_ANALYSIS    = "Analyse des sorties par adhérent"
HELP_MEMBER_ANALYSIS    = " Analyse de la participation aux sorties, randonnées et séjour "
HELP_MEMBER_ANALYSIS   += " par adhérent. Un fichier EXCEL est généré."
BUTT_MEMBER_ANALYSIS    = "Construction du fichier EXCEL"
TEXT_METHOD             = "Choix du type de comptage"

### - Page analyse tendancielle
TEXT_ACTIVITE_PI = "Choix de l'activité"
TEXT_TENDANCE_SORTIES     = "Analyse tendancielle des sorties"
HELP_TENDANCE_SORTIES     = (" La synthèse est effectuée à partir des fichiers"
                             " d'émargement aux sorties, randonnées et séjours")
BUTT_TENDANCE_SORTIES     = "Lancer l'analyse"
TEXT_PRESENCE_EFFECTIF    = "Analyse de la présence an club"
HELP_PRESENCE_EFFECTIF    = (" L'analyse de l'évolution de l'effectif est issue"
                             " des archives disponnibles à partir de 2012.")
BUTT_PRESENCE_EFFECTIF    = "Lancer l'analyse de l'évolution des effectifs"
TEXT_VAE_ANALYSIS         = "Analyse tendancielle depuis 2019 de la population de VAE"
HELP_VAE_ANALYSIS         = (" L'analyse de l'évolution de l'effectif est issue"
                             " des fichiers EXCEL https://ffcyclo.org/")
BUTT_VAE_ANALYSIS         = "Lancer l'analyse de l'évolution des VAE"

### - Page divers
TEXT_GEO_PI               = "Choix de l'année"
TEXT_GEO                  = "Analyse de la répartition géographique des adhérents"
HELP_GEO                  = (" La synthèse est effectuée à partir des fichiers"
                             " EXCEL extraits de https://cyclo38ffct.fr/")
BUTT_GEO                  = "Lancer la construction du fichier html"
TEXT_NBR_SEJOURS          = "Histogramme de nombre de membres ayant participé à N séjours"
HELP_NBR_SEJOURS    = (" La construction de l'histogramme est issue"
                       " de l'anlyse du fichier EXCEL : 'synthese_adherent.xl'.")
BUTT_NBR_SEJOURS = "Lancer la construction de l'histogramme"

### - Page analyse tendancielle
TEXT_SEJOUR               = "Analyse des sejours en termes de cout et de duree"
HELP_SEJOUR               = (" La synthèse est effectuée à partir du fichiers"
                             " EXCEL synthese_adherents.xlsx")
BUTT_SEJOUR               = "Lancer l'analyse"



################# Liste répertoires
DIR_SORTIES_LIST = ['SEJOUR',
                    'SORTIES DE DERNIERE MINUTE',
                    'SORTIES DU DIMANCHE',
                    'SORTIES DU JEUDI',
                    'SORTIES DU SAMEDI',
                    'SORTIES HIVER',
                    'SORTIES VTT']

ACTIVITE_LIST =    ['nbr_participations_sejours',
                    'nbr_jours_participation_sejours',
                    'sortie_dimanche_club',
                    'sortie_samedi_club',
                    'sortie_hiver_club',
                    'sortie_jeudi_club',
                    'randonnee',
                    'nbr_sejours',
                    'nbr_jours_sejours',
                   ]


__all__ = ['create_sorties_analysis']

# Standard library imports
import os
import shutil
import tkinter as tk
from tkinter import font as tkFont
from tkinter import filedialog
from tkinter import messagebox
from pathlib import Path

# 3rd party imports
import pandas as pd

# Local imports
import ctg.ctggui.guiglobals as gg
from ctg.ctgfuncts.ctg_plot import stat_sorties_club
from ctg.ctgfuncts.ctg_synthese import synthese_randonnee
from ctg.ctggui.guitools import encadre_RL
from ctg.ctggui.guitools import font_size
from ctg.ctggui.guitools import mm_to_px
from ctg.ctggui.guitools import place_after
from ctg.ctggui.guitools import place_bellow
from ctg.ctggui.guitools import last_available_years
from ctg.ctggui.guitools import get_available_sorties

def _launch_sorties_analysis(ctg_path,
                             year,
                             type_sortie):
    """
    """



    path_sorties =  ctg_path / Path(str(year)) / Path(type_sortie)
    file_label =  ctg_path / Path(str(year)) / Path(r'DATA/info_randos.xlsx')
    df_total = stat_sorties_club(path_sorties,ctg_path,None,file_label,year)


    #info_title = "- Information -"
    #info_text  = f"L'analyse des IFs a été effectuée pour l'année {year_select} "
    #info_text += f"avec les valeurs {analysis_if}. "
    #info_text += f"\n\nLes fichiers obtenus ont été créés dans le dossier :"
    #info_text += f"\n\n''."
    #info_text += f"\n\nLa base de donnée des indicateurs respective de l'Institut "
    #info_text += f"et de chaque département a été mise à jour "
    #info_text += f"avec les résultats de cette analyse et se trouve dans le dossier :"
    #info_text += f"\n\n''."
    #messagebox.showinfo(info_title, info_text)

def _launch_rando_analysis(ctg_path,
                           year,
                           type_sortie):
    """
    """

    if type_sortie == "SEJOUR":
        synthese_randonnee(year,ctg_path,'SEJOUR')
    else:
        synthese_randonnee(year,ctg_path,'RANDONNEE')

def create_sorties_analysis(self, master, page_name, institute, ctg_path):

    """
    Description : function working as a bridge between the BiblioMeter
    App and the functionalities needed for the use of the app

    Uses the following globals :
    - DIC_OUT_PARSING
    - FOLDER_NAMES

    Args: takes only self and ctg_path as arguments.
    self is the instense in which PageThree will be created
    ctg_path is a type Path, and is the path to where the folders
    organised in a very specific way are stored

    Returns : nothing, it create the page in self
    """

    # Internal functions
    def _launch_sorties_analysis_try():
        # Getting year selection
        year_select = variable_years.get()
        type_sortie = variable_sortie.get()
        _launch_sorties_analysis(ctg_path,
                                 year_select,
                                 type_sortie)
        return

    def _launch_rando_analysis_try():
        # Getting year selection
        year_select = variable_years.get()
        type_sortie = variable_sortie.get()
        _launch_rando_analysis(ctg_path,
                               year_select,
                               type_sortie)
        return

    from ctg.ctggui.pageclasses import AppMain

    # Setting effective font sizes and positions (numbers are reference values)
    eff_etape_font_size      = font_size(gg.REF_ETAPE_FONT_SIZE,   AppMain.width_sf_min)
    eff_launch_font_size     = font_size(gg.REF_ETAPE_FONT_SIZE-1, AppMain.width_sf_min)
    eff_help_font_size       = font_size(gg.REF_ETAPE_FONT_SIZE-2, AppMain.width_sf_min)
    eff_select_font_size     = font_size(gg.REF_ETAPE_FONT_SIZE, AppMain.width_sf_min)
    eff_buttons_font_size    = font_size(gg.REF_ETAPE_FONT_SIZE-3, AppMain.width_sf_min)

    sorties_analysis_x_pos_px     = mm_to_px(10 * AppMain.width_sf_mm,  gg.PPI)
    sorties_analysis_y_pos_px     = mm_to_px(75 * AppMain.height_sf_mm, gg.PPI)
    rando_analysis_x_pos_px       = mm_to_px(10 * AppMain.width_sf_mm,  gg.PPI)
    rando_analysis_y_pos_px       = mm_to_px(120 * AppMain.height_sf_mm, gg.PPI)
    launch_dx_px             = mm_to_px( 0 * AppMain.width_sf_mm,  gg.PPI)
    launch_dy_px             = mm_to_px( 5 * AppMain.height_sf_mm, gg.PPI)

    year_button_x_pos        = mm_to_px(gg.REF_YEAR_BUT_POS_X_MM * AppMain.width_sf_mm,  gg.PPI)
    year_button_y_pos        = mm_to_px(gg.REF_YEAR_BUT_POS_Y_MM * AppMain.height_sf_mm, gg.PPI)
    sorties_button_x_pos     = mm_to_px(gg.REF_SORTIES_BUT_POS_X_MM * AppMain.width_sf_mm,  gg.PPI)
    sorties_button_y_pos     = mm_to_px(gg.REF_SORTIES_BUT_POS_Y_MM * AppMain.height_sf_mm, gg.PPI)
    dy_year                  = -6
    ds_year                  = 5
    dy_sorties               = -6

    # Setting common attributs
    etape_label_format = 'left'
    etape_underline    = -1

    ### Choix de l'année
    list_years = last_available_years(ctg_path,year_number=2021)
    default_year = list_years[-1]
    variable_years = tk.StringVar(self)
    variable_years.set(default_year)

        # Création de l'option button des années
    self.font_OptionButton_years = tkFont.Font(family = gg.FONT_NAME,
                                               size = eff_buttons_font_size)
    self.OptionButton_years = tk.OptionMenu(self,
                                            variable_years,
                                            *list_years)
    self.OptionButton_years.config(font = self.font_OptionButton_years)

        # Création du label
    self.font_Label_years = tkFont.Font(family = gg.FONT_NAME,
                                        size = eff_select_font_size,
                                        weight = 'bold')
    self.Label_years = tk.Label(self,
                                text = gg.TEXT_YEAR_PI,
                                font = self.font_Label_years)
    self.Label_years.place(x = year_button_x_pos, y = year_button_y_pos)

    place_after(self.Label_years, self.OptionButton_years, dy = dy_year)

    ### Choix de du type de sortie
    list_sorties = get_available_sorties(ctg_path,variable_years.get())
    default_sortie = list_sorties[-1]
    variable_sortie = tk.StringVar(self)
    variable_sortie.set(default_sortie)

        # Création de l'option button des sorties
    self.font_OptionButton_sorties = tkFont.Font(family = gg.FONT_NAME,
                                                 size = eff_buttons_font_size)
    self.OptionButton_sorties = tk.OptionMenu(self,
                                              variable_sortie,
                                              *list_sorties)
    self.OptionButton_sorties.config(font = self.font_OptionButton_sorties)

        # Création du label
    self.font_Label_sorties = tkFont.Font(family = gg.FONT_NAME,
                                        size = eff_select_font_size,
                                        weight = 'bold')
    self.Label_sorties = tk.Label(self,
                                  text = gg.TEXT_SORTIES_PI,
                                  font = self.font_Label_sorties)
    self.Label_sorties.place(x = sorties_button_x_pos, y = sorties_button_y_pos)

    place_after(self.Label_sorties, self.OptionButton_sorties, dy = dy_sorties)

    ################## Analyse des sorties

    ### Titre
    sorties_analysis_font = tkFont.Font(family = gg.FONT_NAME,
                                        size = eff_etape_font_size,
                                        weight = 'bold')
    sorties_analysis_label = tk.Label(self,
                                      text = gg.TEXT_SORTIES,
                                      justify = etape_label_format,
                                      font = sorties_analysis_font,
                                      underline = etape_underline)

    sorties_analysis_label.place(x = sorties_analysis_x_pos_px,
                                 y = sorties_analysis_y_pos_px)

    ### Explication
    help_label_font = tkFont.Font(family = gg.FONT_NAME,
                                  size = eff_help_font_size)
    help_label = tk.Label(self,
                          text = gg.HELP_SORTIES,
                          justify = "left",
                          font = help_label_font)
    place_bellow(sorties_analysis_label,
                 help_label)

    ### Bouton pour lancer l'analyse des sorties
    sorties_analysis_launch_font = tkFont.Font(family = gg.FONT_NAME,
                                          size = eff_launch_font_size)
    sorties_analysis_launch_button = tk.Button(self,
                                               text = gg.BUTT_SORTIES,
                                               font = sorties_analysis_launch_font,
                                               command = lambda: _launch_sorties_analysis_try())
    place_bellow(help_label,
                 sorties_analysis_launch_button,
                 dx = launch_dx_px,
                 dy = launch_dy_px)

    ################## Analyse des randonnées

    ### Titre
    rando_analysis_font = tkFont.Font(family = gg.FONT_NAME,
                                      size = eff_etape_font_size,
                                      weight = 'bold')
    rando_analysis_label = tk.Label(self,
                                    text = gg.TEXT_RANDO,
                                    justify = etape_label_format,
                                    font = rando_analysis_font,
                                    underline = etape_underline)

    rando_analysis_label.place(x = rando_analysis_x_pos_px,
                                 y = rando_analysis_y_pos_px)

    ### Explication
    help_label_font = tkFont.Font(family = gg.FONT_NAME,
                                  size = eff_help_font_size)
    help_label_rando = tk.Label(self,
                                text = gg.HELP_RANDO,
                                justify = "left",
                                font = help_label_font)
    place_bellow(rando_analysis_label,
                 help_label_rando)

    ### Bouton pour lancer l'analyse des sorties
    rando_analysis_launch_font = tkFont.Font(family = gg.FONT_NAME,
                                             size = eff_launch_font_size)
    rando_analysis_launch_button = tk.Button(self,
                                             text = gg.BUTT_RANDO,
                                             font = sorties_analysis_launch_font,
                                             command = lambda: _launch_rando_analysis_try())
    place_bellow(help_label_rando,
                 rando_analysis_launch_button,
                 dx = launch_dx_px,
                 dy = launch_dy_px)


__author__ = 'F. Bertin'
__license__ = "MIT"
__version__ = '1.1.3'

from ctg.ctgfuncts.ctg_plot import *
from ctg.ctgfuncts.ctg_effectif import *
from ctg.ctgfuncts.ctg_synthese import *
from ctg.ctgfuncts.ctg_classes import *
from ctg.ctgfuncts.ctg_tools import *
from ctg.ctgfuncts.ctg_cd import *


__all__ =['evolution_sorties',
          'nbr_sejours_adherent',
          'plot_pie_synthese',
          'stat_cout_sejour',
          'synthese',
          'synthese_adherent',
          'synthese_randonnee',
          ]

# Standard library imports
import datetime
import pathlib
import os
import os.path
from collections import Counter
from collections import namedtuple
from pathlib import Path
from tkinter import messagebox
from typing import Optional

# Third party imports
import matplotlib.pyplot as plt
import numpy as np
import pandas as pd
import yaml

# Internal imports
from ctg.ctggui.guiglobals import ACTIVITE_LIST
from ctg.ctgfuncts.ctg_classes import EffectifCtg
from ctg.ctgfuncts.ctg_effectif import read_effectif_corrected
from ctg.ctgfuncts.ctg_tools import get_sejour_info
from ctg.ctgfuncts.ctg_tools import get_cout_total
from ctg.ctgfuncts.ctg_tools import normalize_tag

def synthese(year:str,ctg_path:pathlib.WindowsPath)->None:

    path_dir_list = [ctg_path / Path(year) / Path('SORTIES DU SAMEDI') / Path('EXCEL'),
                     ctg_path / Path(year) / Path('SORTIES DU DIMANCHE') / Path('EXCEL'),
                     ctg_path / Path(year) / Path('SORTIES DU JEUDI') / Path('EXCEL'),
                     ctg_path / Path(year) / Path('SORTIES HIVER') / Path('EXCEL'),
                     ctg_path / Path(year) / Path('SORTIES DU LUNDI')/ Path('EXCEL'),
                     ctg_path / Path(year) / Path('SEJOUR') / Path('EXCEL')]

    list_df = []
    for path_dir in path_dir_list:
        if os.path.isdir(path_dir):
            files = [x for x in os.listdir(path_dir) if x.endswith('.xlsx') and "~$" not in x]
            list_df.extend([pd.read_excel(path_dir / Path(file), engine='openpyxl')
                           for file in files])

    df_total = pd.concat(list_df, ignore_index=True)

    df_effectif = read_effectif_corrected(ctg_path)

    df_total['Pratique VAE'].fillna('Non',inplace=True)

    # nombre moyen de participant par activité
    for x in df_total.groupby(['Type']):
        print(x[0],len(x[1]),len(set(x[1]['sejour'])),len(x[1])/len(set(x[1]['sejour'])))

    file = ctg_path / Path(year) / Path('STATISTIQUES') / Path('EXCEL') / Path('synthese.xlsx')
    df_total.to_excel(file)

def plot_pie_synthese(year:str,ctg_path:pathlib.WindowsPath,mode: Optional[bool] = False)->None:

    '''Plot from the EXCEL file `synthese.xlsx` the pie plot of 
    the number of participation to the evenments'''

    def func(pct, allvalues):
        absolute = round(pct / 100.*np.sum(allvalues),0)
        #return "{:.1f}%\n({:d})".format(pct, absolute)
        return  f"{int(round(absolute,1))}\n{round(pct,1)} %"
    
    file_in = ctg_path / Path(year) / Path('STATISTIQUES') / Path('EXCEL') / Path('synthese.xlsx')
    df_total = pd.read_excel(file_in)
    df_total = df_total.dropna(subset=['Type'])
    df_total = df_total.dropna(subset=['Nom'])
    
    if mode: # un sejour vaut pour un jour 
        df_total['nbr_jours'] = 1

    dagg = df_total.groupby('Type')['nbr_jours'].agg('sum')



    explode_dic = {'RANDONNEE':0.1,
                   'SEJOUR':0.0,
                   'SORTIES DU DIMANCHE':0.2,
                   'SORTIES DU JEUDI':0.2,
                   'SORTIES DU LUNDI':0.2,
                   'SORTIES DU SAMEDI':0.2,
                   'SORTIES HIVER':0.2}

    data = []
    sorties = []
    for type_sejour, nbr in zip(dagg.index,dagg.tolist()):
        if nbr!=0:
            data.append(nbr)
            sorties.append(type_sejour)

    explode = [explode_dic[typ] for typ in sorties]


    # Creating plot
    #fig = plt.figure(figsize =(10, 7))
    _, _, autotexts = plt.pie(data,
                              labels = sorties,
                              autopct = lambda pct: func(pct, data),
                              explode = explode,
                              textprops={'fontsize': 18})
                              
    title = f'{year} (Nombre total de participations: {sum(data)})'
    plt.title(title, pad=50, fontsize=20)

    _ = plt.setp(autotexts, **{'color':'k', 'weight':'bold', 'fontsize':14})

    plt.tight_layout()
    plt.show()

    fig_file = 'SORTIES_PIE.png'
    plt.savefig(ctg_path / Path(year) / Path('STATISTIQUES/IMAGE') / Path(fig_file),bbox_inches='tight')



def synthese_adherent(year:str,ctg_path:pathlib.WindowsPath):

    '''generates the EXCEL file 'synthese_adherent.xlsx' with 11 columns :'Nom','Prénom',
    'SORTIE DU DIMANCHE CLUB','SORTIE DU SAMEDI CLUB','SORTIE DU JEUDI CLUB',
    'RANDONNEE','SEJOUR-JOUR','Nbr_SEJOURS', 'SORTIE HIVER','TOTAL','COUT_SEJOUR',
    '''

    file_in = ctg_path / Path(year) / Path('STATISTIQUES') / Path('EXCEL') / Path('synthese.xlsx')
    df_total = pd.read_excel(file_in)
    df_total = df_total.dropna(subset=['Type'])
    nbre = {}

    for id_licence, dg in df_total.groupby('N° Licencié'):

        nbre[id_licence]=[dg['Nom'].unique()[0],dg['Prénom'].unique()[0]]

        nb_sortie_dimanche = len(dg.query("Type.str.contains('SORTIES DU DIMANCHE')"))
        nbre[id_licence] = nbre[id_licence] + [nb_sortie_dimanche]

        nb_sortie_samedi = len(dg.query("Type.str.contains('SORTIES DU SAMEDI')"))
        nbre[id_licence] = nbre[id_licence] + [nb_sortie_samedi]

        nb_sortie_jeudi = len(dg.query("Type.str.contains('SORTIES DU JEUDI')"))
        nbre[id_licence] = nbre[id_licence] + [nb_sortie_jeudi]

        nb_rando = len(dg.query("Type.str.contains('RANDONNEE')"))
        nbre[id_licence] = nbre[id_licence] + [nb_rando]

        nb_sejour_jours = sum(dg.query("Type.str.contains('SEJOUR')")['nbr_jours'].tolist())
        nbre[id_licence] = nbre[id_licence] + [nb_sejour_jours]

        nb_sejour = len(dg.query("Type.str.contains('SEJOUR')")['sejour'].unique())
        nbre[id_licence] = nbre[id_licence] + [nb_sejour]

        nb_hiver = len(dg.query("Type.str.contains('SORTIES HIVER')"))
        nbre[id_licence] = nbre[id_licence] + [nb_hiver]

        nbr_evenements = [nb_sortie_dimanche +
                          nb_sortie_samedi +
                          nb_sortie_jeudi +
                          nb_rando +
                          nb_sejour_jours +
                          nb_hiver]
        nbre[id_licence] = nbre[id_licence] + nbr_evenements

        dg['cout_sejour'] = dg['cout_sejour'].fillna(0)
        nbre[id_licence] = nbre[id_licence] + [sum(dg['cout_sejour'])]


    dg = pd.DataFrame.from_dict(nbre).T
    dg.columns = [
                 'Nom',
                 'Prénom',
                 'SORTIE DU DIMANCHE CLUB',
                 'SORTIE DU SAMEDI CLUB',
                 'SORTIE DU JEUDI CLUB',
                 'RANDONNEE',
                 'SEJOUR-JOUR',
                 'Nbr_SEJOURS',
                 'SORTIE HIVER',
                 'TOTAL',
                 'COUT_SEJOUR',
                 ]

    effectif = EffectifCtg(year,ctg_path)
    df_effectif = effectif.effectif
    df_effectif.index = df_effectif['N° Licencié']
    orphan = set(df_effectif.index) - set(dg.index)
    df_orphan = df_effectif.loc[list(orphan)][['Nom','Prénom']]
    df_orphan[['SORTIE DU DIMANCHE CLUB',
               'SORTIE DU SAMEDI CLUB',
               'SORTIE DU JEUDI CLUB',
               'RANDONNEE',
               'SEJOUR-JOUR',
               'Nbr_SEJOURS',
               'SORTIE HIVER',
               'TOTAL',
               'COUT_SEJOUR',]] = [0,0,0,0,0,0,0,0,0]

    dg = pd.concat([dg, df_orphan], axis=0)

    file_out = ctg_path / Path(year) / Path('STATISTIQUES') / Path('EXCEL')
    file_out = file_out / Path('synthese_adherent.xlsx')
    dg.to_excel(file_out)

def synthese_randonnee(year:str,ctg_path:pathlib.WindowsPath,type_sejour:str):

    '''Creates a plot synthetizing the participation at th randonnées.
    '''

    def addlabels(x,y,offset,y_val=None):
        for i in range(len(x)):
            if y[i] != 0:
                if y_val is None:
                    plt.text(x[i]-0.25,y[i]+offset,round(y[i],1),size=10)
                else:
                    plt.text(x[i]-0.25,y[i]+offset,round(y_val[i],1),size=10)


    file_in = Path(ctg_path) / Path(year) / Path('STATISTIQUES')
    file_in = file_in / Path('EXCEL') / Path('synthese.xlsx')
    df_total = pd.read_excel(file_in)
    df_total = df_total.dropna(subset=['Nom'])
    df_total = df_total.query('Type==@type_sejour')
    dg = df_total.groupby('sejour').agg('count')['N° Licencié']
    
    file_info = Path(ctg_path) / Path(year) / Path('DATA') / Path('info_randos.xlsx')
    info_df = pd.read_excel(file_info)
    type_sejour_m = type_sejour.lower()
    info_df=info_df.query('type==@type_sejour_m')
    info_dic = dict(zip(info_df['date'],info_df['name_activite']))
    info_duree = dict(zip(info_df['date'],info_df['nbr_jours']))    
    
    tag_list = [normalize_tag(x,year) for x in dg.index]
    labelx = [f'{k[3:8]} {info_dic[k].strip()}' for k in tag_list]
    duree_sejour = [info_duree[k] for k in tag_list]

    cout_total_rando = get_cout_total(year,type_sejour.lower(),dg,ctg_path)

    fig = plt.figure()
    plt.bar(range(len(dg)),dg.tolist())
    addlabels(list(range(len(dg))),dg.tolist(),0.2)
    plt.xticks(range(len(dg)), labelx, rotation='vertical')
    plt.tick_params(axis='x', rotation=90, labelsize=10)
    plt.tick_params(axis='y',labelsize=10)

    if type_sejour == 'RANDONNEE':
        long_string = (f'Année : {year}\n'
                       f'# randos : {len(dg)} , '
                       f'# participants : '
                       f'{sum(dg)}, Coût : {cout_total_rando} €')
        _ = plt.title(long_string)
    else:
        y = [1] *  len(duree_sejour)
        addlabels(list(range(len(dg))),y,1,duree_sejour)
        long_string = (f'Année : {year}\n'
                       f'# jours : {sum(duree_sejour)} '
                       f'# sejours : {len(dg)} , '
                       f'# participants : {sum(dg)}'
                       f', Coût : {cout_total_rando} €')
        _ = plt.title(long_string)
    plt.tight_layout()
    plt.show()

def nbr_sejours_adherent(year:str, ctg_path:pathlib.WindowsPath):

    '''Generates the histgramm of the number of sejour versus the number of members
    '''

    plt.style.use('ggplot')

    # function to add value labels
    def addlabels(x,y):
        for i in range(len(x)):
            plt.text(x[i]-0.2,y[i]+1,y[i],size=label_size)
    label_size = 15
    file_in = ctg_path / Path(year) / Path('STATISTIQUES') / Path('EXCEL')
    file_in = file_in / Path('synthese_adherent.xlsx')
    df_total = pd.read_excel(file_in)

    c = Counter()
    c = Counter(df_total['Nbr_SEJOURS'].tolist())
    c = c.most_common()

    x, y = zip(*c)
    x = list(x)
    y = list(y)

    fig, ax = plt.subplots(figsize=(5,5))
    plt.bar([str(x_s) for x_s in x],y)
    plt.xlabel('Nombre de participation à des séjours')
    plt.ylabel('Nombre de licenciers')
    plt.tick_params(axis='x', labelsize=label_size)
    plt.tick_params(axis='y', labelsize=label_size)
    plt.title(year,pad=20, fontsize=20)

    ax.set_xlabel('N séjours', fontsize = label_size)
    ax.set_ylabel('Nombre de CTG ayant \n participé à N séjours', fontsize = label_size)

    x.sort()
    addlabels(x,y)
    plt.tight_layout()
    plt.show()

    fig_file = 'SEJOURS_STAT_PARTICIPATION.png'
    plt.savefig(ctg_path / Path(year) / Path('STATISTIQUES/IMAGE') / Path(fig_file),bbox_inches='tight')

def _read_memory_sorties()->dict:

    '''Reads the default PVcharacterization.yaml config file'''

    parent = Path(__file__).parent.parent
    path_config_file = parent  / Path('ctgfuncts/CTG_RefFiles') / Path('memory_sorties.yml')
    with open(path_config_file) as file:
        memory = yaml.safe_load(file)

    return memory

def evolution_sorties(type:str,ctg_path:pathlib.WindowsPath):

    def add_memory(stat_dic,years):
        
        '''Add years from 2014 to 2021to the dic `stat_year`. These statistics are stored
        in the package and can only be modified by the package owner by pulling a request at
        https://github.com/Bertin-fap/ctgutils.
        '''
        
        memory = _read_memory_sorties()


        for year,v in memory['memory'].items():

            stat_dic[str(year)] = statyear(v['PARTICIPATION_SEJOURS'], # participants sejour
                                           v['Nombre_sorties_sejour'], # jours_sejour
                                           v['SORTIES_CLUB_DIMANCHE'], # sortie_dimanche_club
                                           v['SORTIES_CLUB_SAMEDI'],   # sortie_samedi_club
                                           v['SORTIES_HIVER'],         # sortie_hiver_club
                                           v['SORTIES_CLUB_JEUDI'],    # sortie_jeudi_club
                                           v['RANDONNEES'],            # randonnee
                                           v['Nombre_sejours'],        # nbr_sejours
                                           v['Nombre_jours_sejour'],)  # nbr_jours_sejours

            years.append(str(year))

    def fill_stat_year(year:str):
    
        '''Builds the dict stat_year using the EXCEL file 
        '''

        sejour_info = get_sejour_info(ctg_path,year)

        file_in = ctg_path / Path(str(year)) / Path('STATISTIQUES') / Path('EXCEL')
        file_in = file_in / Path('synthese_adherent.xlsx')

        df = pd.read_excel(file_in)

        stat_year = statyear(df['Nbr_SEJOURS'].sum(),             # nbr_jours_participation_sejours
                             df['SEJOUR-JOUR'].sum(),             # nbr_participations_sejour
                             df['SORTIE DU DIMANCHE CLUB'].sum(), # sortie_dimanche_club
                             df['SORTIE DU SAMEDI CLUB'].sum(),   # sortie_samedi_club
                             df['SORTIE HIVER'].sum(),            # sortie_hiver_club
                             df['SORTIE DU JEUDI CLUB'].sum(),    # sortie_jeudi_club
                             df['RANDONNEE'].sum(),               # randonnee
                             sejour_info.nbr_sejours,             # nbr_sejours
                             sejour_info.nbr_jours)               # nbr_jours_sejours

        return stat_year

    def addlabels(x,y,offset):
        for i in range(len(x)):
            plt.text(x[i],y[i]+offset,round(y[i],1),size=10)

    def plot_stat(years,nb_participants,title,label_y):
        plt.figure(figsize=(8, 6))
        colors = ['#fdaa48']
        size_label = 20
        plt.bar(years,nb_participants,color=colors)
        plt.ylabel(label_y,size=size_label)
        addlabels(years,nb_participants,1)
        plt.xticks(rotation=90)
        plt.tick_params(axis='x', labelsize=size_label)
        plt.tick_params(axis='y', labelsize=size_label)
        plt.title(title,fontsize=18)
        plt.tight_layout()
        plt.show()

    statyear = namedtuple('activite', ACTIVITE_LIST)

    plt.style.use('ggplot')
    years = []
    stat_dic = {}
    add_memory(stat_dic,years)

    today = datetime.datetime.now()
    years_new = [str(year) for year in range(2022,today.year+1)]
    
    for year in years_new:
        stat_dic[year] = fill_stat_year(year)

    years = years + years_new


    if type == 'nbr_jours_participation_sejours':
        plot_stat(years,
                  [stat_dic[year].nbr_jours_participation_sejours for year in years],
                  type,
                  type)
    elif  type == 'sortie_dimanche_club':
        plot_stat(years,
                  [stat_dic[year].sortie_dimanche_club for year in years],
                  type,
                  '#participants')
    elif  type == 'sortie_samedi_club':
        plot_stat(years,
                  [stat_dic[year].sortie_samedi_club for year in years],
                  type,
                  '#participants')
    elif  type == 'sortie_jeudi_club':
        plot_stat(years,
                  [stat_dic[year].sortie_jeudi_club for year in years],
                  type,
                  '#participants')
    elif  type == 'randonnee':
        plot_stat(years,
                  [stat_dic[year].randonnee for year in years],
                  type,
                  '#participants')
    elif  type == 'nbr_participations_sejours':
        plot_stat(years,
                  [stat_dic[year].nbr_participations_sejours for year in years],
                  'Nombre de participations aux séjours',
                  '# participations aux séjours')
    elif  type == 'nbr_sejours':
        plot_stat(years,
                  [stat_dic[year].nbr_sejours for year in years],
                  'Nombre de séjours',
                  '# séjours')
    elif  type == 'nbr_jours_sejours':
        plot_stat(years,
                  [stat_dic[year].nbr_jours_sejours for year in years],
                  'Nombre de jours séjours',
                  '# jours séjour')
    elif type == 'synthese':
        plot_synthese_sortie(stat_dic)
        

def stat_cout_sejour(year:str,ctg_path:pathlib.WindowsPath)->None:

    '''Builds the histogramm of the number m of members whose have take part to n sejours.
    '''

    def plot_histo(col_name,idx_plot,labelx,labely,unit):
        df.hist(column=col_name,
                bins=80,
                ax=ax[idx_plot],
                xlabelsize=15,
                ylabelsize=15,)
        ax[idx_plot].set_xlabel(labelx,fontsize=18)
        ax[idx_plot].set_ylabel(labely,fontsize=18)

        col_without_zero = [x for x in df[col_name] if x>0]

        mean_col = round(np.mean(df[col_name]),1)
        mean_col_without_zero = round(np.mean(col_without_zero),1)
        med_col = round(np.median(df[col_name]),1)
        med_col_without_zero = round(np.median(col_without_zero),1)
        long_string = (f'     Total : {sum(df[col_name])} {unit}\n'
                       f'     Moyenne : {mean_col} {unit}\n'
                       f'     Mediane : {med_col} {unit}\n'
                       'Sans prendre en compte la classe 0 :\n'
                       f'     Moyenne : {mean_col_without_zero} {unit}\n'
                       f'     Mediane : {med_col_without_zero} {unit}\n\n')
        return long_string

    fig, ax = plt.subplots(nrows=1, ncols=2, sharey=True)

    file = ctg_path / Path(year) / Path('STATISTIQUES') / Path('EXCEL') 
    file = file / Path('synthese_adherent.xlsx')
    if not os.path.isfile(file):
        synthese_adherent(year,ctg_path)

    df = pd.read_excel(file)

    file = ctg_path / Path(year) / Path('DATA') / Path('info_randos.xlsx')
    info_df = pd.read_excel(file)
    nbr_sejours = info_df.query('type=="sejour"')['date'].count()
    nbr_jours = sum([x for x in info_df["nbr_jours"] if x>1])

    comment = f'Année : {year}\n'
    comment += f'Nombre de séjours : {nbr_sejours}\n'
    comment += 'Analyse du coût des séjours :\n'
    comment_cout = plot_histo('COUT_SEJOUR',0, '€','# membres','€')
    comment += f'     Cout annuel des séjours : {sum(info_df["Cout"].fillna(0))} €\n'
    comment += comment_cout
    comment += 'Analyse de la durée des séjours :\n'
    comment += f'     Durée totale des séjours : {nbr_jours} jours\n'
    comment_jour = plot_histo('SEJOUR-JOUR',1,'# jours séjour','','jours')
    comment += comment_jour
    messagebox.showinfo('info sejour', comment)
    fig.suptitle(f'Année : {str(year)}',fontsize=20)
    plt.tight_layout()
    plt.show()
    file = ctg_path / Path(year) / Path('STATISTIQUES') / Path('TEXT')
    file = file / Path('synthese_sejour.txt')
    with open(file,'w', encoding='utf-8') as f:
        f.write(comment)

def plot_synthese_sortie(stat_dic:dict):
    
    '''Synthetic plot of the events
    '''
    df = pd.DataFrame.from_dict(stat_dic).T
    df.columns = ['PARTICIPATION_SEJOURS',
                  'Nombre_sorties_sejour',
                  'SORTIES_CLUB_DIMANCHE',
                  'SORTIES_CLUB_SAMEDI',
                  'SORTIES_HIVER',      
                  'SORTIES_CLUB_JEUDI', 
                  'RANDONNEES',         
                  'Nombre_sejours',     
                  'Nombre_jours_sejour']
    df = df.drop(['SORTIES_HIVER','Nombre_sejours',],axis=1)
    df['total'] = (df['RANDONNEES']+
                   df['Nombre_sorties_sejour']+ 
                   df['SORTIES_CLUB_DIMANCHE']+
                   df['SORTIES_CLUB_JEUDI']+
                   df['SORTIES_CLUB_SAMEDI'])
    
    plt.style.use('ggplot')
    fig, (ax1, ax2) = plt.subplots(2, 1, sharex=True,figsize=(15, 5))
    fig.subplots_adjust(hspace=0.05) 
    
    ax2.plot(df.index[0:6],df['RANDONNEES'][0:6], '-*b',label = 'RANDONNEES')
    ax2.plot(df.index[8:12],df['RANDONNEES'][8:],'-*b')
    ax2.plot(df.index[0:6],df['PARTICIPATION_SEJOURS'][0:6],'r-',label='PARTICIPATION_SEJOURS',linewidth=3)
    ax2.plot(df.index[8:12],df['PARTICIPATION_SEJOURS'][8:],'r-',linewidth=3)
    ax1.plot(df.index[0:6],df['Nombre_sorties_sejour'][0:6],'r-',label='Nombre_sorties_sejour',linewidth=3)
    ax1.plot(df.index[8:12],df['Nombre_sorties_sejour'][8:],'r-',linewidth=3)    
    ax2.plot(df.index[0:6],df['SORTIES_CLUB_DIMANCHE'][0:6],'*-.k',label='SORTIES_CLUB_DIMANCHE')
    ax2.plot(df.index[8:12],df['SORTIES_CLUB_DIMANCHE'][8:],'*-.k')
    ax2.plot(df.index[1:6],df['SORTIES_CLUB_JEUDI'][1:6],'-..m',label='SORTIES_CLUB_JEUDI')
    ax2.plot(df.index[8:12],df['SORTIES_CLUB_JEUDI'][8:],'-..m')
    ax2.plot(df.index[0:6],df['SORTIES_CLUB_SAMEDI'][0:6],'-+g',label='SORTIES_CLUB_SAMEDI')
    ax2.plot(df.index[8:12],df['SORTIES_CLUB_SAMEDI'][8:],'-+g')
    
    ax1.plot(df.index[0:6],df['total'][0:6],'-+y',label='total')
    ax1.plot(df.index[8:12],df['total'][8:],'-+y')
    ax2.legend(loc='upper right', bbox_to_anchor=(1.25, 1))
    ax1.legend(loc='upper right', bbox_to_anchor=(1.25, 1))
    plt.xticks(rotation=90)
    ax1.tick_params(axis='both', which='major', labelsize=16)
    ax2.tick_params(axis='both', which='major', labelsize=16)
   
    ax1.set_ylabel('# participations', fontsize=16)
    ax2.set_ylabel('# participations', fontsize=16)
    
    ax1.set_ylim(1250, 3000)
    ax2.set_ylim(50, 650)
    
    ax1.spines.bottom.set_visible(False)
    ax2.spines.top.set_visible(False)
    ax1.xaxis.tick_top()
    ax2.xaxis.tick_bottom()
    
    d = .5  # proportion of vertical to horizontal extent of the slanted line
    kwargs = dict(marker=[(-1, -d), (1, d)], markersize=12,
                  linestyle="none", color='k', mec='k', mew=1, clip_on=False)
    ax1.plot([0, 1], [0, 0], transform=ax1.transAxes, **kwargs)
    ax2.plot([0, 1], [1, 1], transform=ax2.transAxes, **kwargs)
    plt.tight_layout()
    plt.show()


__all__ = ['encadre_RL',
           'encadre_UD',
           'font_size',
           'get_available_sorties',
           'last_available_years',
           'mm_to_px',
           'place_after',
           'place_bellow',
           'place_bellow_LabelEntry',
           'str_size_mm',
           'show_frame',
           'create_archi',
           'window_properties'
          ]

# Standard library imports
import datetime
import itertools
import os
import re
from calendar import monthrange
from pathlib import Path
from tkinter import messagebox

# Standard library imports
import math

# Local imports
import ctg.ctggui.guiglobals as gg
from ctg.ctggui.guiglobals import IN_TO_MM
from ctg.ctggui.guiglobals import DIR_SORTIES_LIST


def show_frame(self, page_name):
    '''Show a frame for the given page name'''
    frame = self.frames[page_name]
    frame.tkraise()


def last_available_years(ctg_path, year_number=2021):

    '''
    Returns a list of the available five last available years where corpuses are stored
    '''

    # Récupérer les corpus disponibles TO DO : consolider le choix des années
    try:
        list_dir = os.listdir(ctg_path)
        years_full_list = list()
        for year in list_dir:
            if re.findall(r'^\d{4}$',year):
                years_full_list.append(year)
        if year_number is not None:
            years_list = [year for year in years_full_list if int(year)>year_number]
    except FileNotFoundError:
        warning_title = "!!! ATTENTION : Dossier de travail inaccessible !!!"
        warning_text  = f"L'accès au dossier {ctg_path} est impossible."
        warning_text += f"\nChoisissez un autre dossier de travail."
        messagebox.showwarning(warning_title, warning_text)
        years_list = []
    return years_list

def place_after(gauche, droite, dx = 5, dy = 0):
    gauche_info = gauche.place_info()
    x = int(gauche_info['x']) + gauche.winfo_reqwidth() + dx
    y = int(gauche_info['y']) + dy
    droite.place(x = x, y = y)

def place_bellow(haut, bas, dx = 0, dy = 5):
    haut_info = haut.place_info()
    x = int(haut_info['x']) + dx
    y = int(haut_info['y']) + haut.winfo_reqheight() + dy
    bas.place(x = x, y = y)

def encadre_RL(fond, gauche, droite, color = "red", dn = 10, de = 10, ds = 10, dw = 10):

    gauche_info = gauche.place_info()
    droite_info = droite.place_info()

    x1 = int(gauche_info['x']) - dw
    y1 = int(gauche_info['y']) - dn

    x2 = int(gauche_info['x']) + gauche.winfo_reqwidth() + droite.winfo_reqwidth() + de
    y2 = int(droite_info['y']) + max(gauche.winfo_reqheight(), droite.winfo_reqheight()) + ds

    rectangle = fond.create_rectangle(x1, y1, x2, y2, outline = color, width = 2)
    fond.place(x = 0, y = 0)

def encadre_UD(fond, haut, bas, color = "red", dn = 10, de = 10, ds = 10, dw = 10):

    haut_info = haut.place_info()
    bas_info = bas.place_info()

    x1 = int(haut_info['x']) - dw
    y1 = int(haut_info['y']) - dn

    x2 = int(bas_info['x']) + max(haut.winfo_reqwidth(), bas.winfo_reqwidth()) + de
    y2 = int(bas_info['y']) + haut.winfo_reqheight() + bas.winfo_reqheight() + ds

    rectangle = fond.create_rectangle(x1, y1, x2, y2, outline = color, width = 2)
    fond.place(x = 0, y = 0)

def place_bellow_LabelEntry(haut, label_entry, dx = 0, dy = 5):

    haut_info = haut.place_info()
    x = int(haut_info['x']) + dx
    y = int(haut_info['y']) + haut.winfo_reqheight() + dy
    label_entry.place(x = x, y = y)


def font_size(size, scale_factor):
    '''Set the fontsize based on scale_factor.
    If the fontsize is less than minimum_size,
    it is set to the minimum size.'''

    fontsize = int(size * scale_factor)
    if fontsize < 8:
        fontsize = 8
    return fontsize


def str_size_mm(text, font, ppi):
    '''The function `_str_size_mm` computes the sizes in mm of a string.

    Args:
        text (str): the text of which we compute the size in mm.
        font (tk.font): the font of the text.
        ppi (int): pixels per inch of the display.

    Returns:
        `(tuple)`: width in mm `(float)`, height in mm `(float)`.

    Note:
        The use of this function requires a tkinter window availability
        since it is based on a tkinter font definition.

    '''

    (w_px,h_px) = (font.measure(text),font.metrics("linespace"))
    w_mm = w_px * IN_TO_MM / ppi
    h_mm = h_px * IN_TO_MM / ppi

    return (w_mm,h_mm)


def mm_to_px(size_mm, ppi, fact = 1.0):
    '''The `mm_to_px' function converts a value in mm to a value in pixels
    using the ppi of the used display and a factor fact.

    Args:
        size_mm (float): value in mm to be converted.
        ppi ( float): pixels per inch of the display.
        fact (float): factor (default= 1).

    Returns:
        `(int)`: upper integer value of the conversion to pixels

    '''

    size_px = math.ceil((size_mm * fact / IN_TO_MM) * ppi)

    return size_px


def window_properties(screen_width_px, screen_height_px):

    # Getting number of pixels per inch screen resolution from imported global DISPLAYS
    ppi = gg.DISPLAYS[gg.BM_GUI_DISP]["ppi"]

    # Setting screen effective sizes in mm from imported global DISPLAYS
    screen_width_mm  = gg.DISPLAYS[gg.BM_GUI_DISP]["width_mm"]
    screen_height_mm = gg.DISPLAYS[gg.BM_GUI_DISP]["height_mm"]

    # Setting screen reference sizes in pixels and mm 
    #from globals internal to module "Coordinates.py"
    ref_width_px  = gg.REF_SCREEN_WIDTH_PX
    ref_height_px = gg.REF_SCREEN_HEIGHT_PX
    ref_width_mm  = gg.REF_SCREEN_WIDTH_MM
    ref_height_mm = gg.REF_SCREEN_HEIGHT_MM

    # Setting secondary window reference sizes in mm 
    #from globals internal to module "Coordinates.py"
    ref_window_width_mm  = gg.REF_WINDOW_WIDTH_MM
    ref_window_height_mm = gg.REF_WINDOW_HEIGHT_MM

    # Computing ratii of effective screen sizes to screen reference sizes in pixels
    scale_factor_width_px  = screen_width_px / ref_width_px
    scale_factor_height_px = screen_height_px / ref_height_px

    # Computing ratii of effective screen sizes to screen reference sizes in mm
    scale_factor_width_mm  = screen_width_mm / ref_width_mm
    scale_factor_height_mm = screen_height_mm / ref_height_mm

    # Computing secondary window sizes in pixels depending on scale factors
    win_width_px  = mm_to_px(ref_window_width_mm * scale_factor_width_mm, ppi)
    win_height_px = mm_to_px(ref_window_height_mm * scale_factor_height_mm, ppi)

    sizes_tuple = (win_width_px, win_height_px,
                   scale_factor_width_px, scale_factor_height_px,
                   scale_factor_width_mm, scale_factor_height_mm)

    return sizes_tuple

def create_folder(root_path, folder, verbose = False):

    folder_path = root_path / Path(folder)
    if not os.path.exists(folder_path):
        os.makedirs(folder_path)
        message = f"{folder_path} created"
    else:
        message = f"{folder_path} already exists"
    if verbose : print(message)
    return folder_path

def create_archi(ctg_path, year_folder, verbose = False):
    '''The `create_archi` function creates a corpus folder with the required architecture.
    It uses the global "ARCHI_YEAR" for the names of the sub_folders.

    Args:
        ctg_path (path): The full path of the working folder.
        year_folder (str): The name of the folder of the corpus.

    Returns:
        (str): The message giving the folder creation status.

    '''

    dir_list = ['DATA','SEJOUR', 'SORTIES DE DERNIERE MINUTE',
                'SORTIES DU DIMANCHE', 'SORTIES DU JEUDI',
                'SORTIES DU SAMEDI','SORTIES HIVER',
                'SORTIES VTT', 'STATISTIQUES']

    new_year_folder_path = create_folder(ctg_path, year_folder, verbose = verbose)
    for dir in dir_list:
        _ = create_folder(new_year_folder_path, dir, verbose = verbose)
        
    for dir in ['EXCEL','HTML', 'IMAGE', 'TEXT']:
        _ = create_folder(Path(ctg_path) / Path(year_folder) / Path('STATISTIQUES'),
                          dir,
                          verbose = verbose)    

    dir_list_sorties = ['SEJOUR', 'SORTIES DE DERNIERE MINUTE',
                        'SORTIES DU DIMANCHE', 'SORTIES DU JEUDI',
                        'SORTIES DU SAMEDI','SORTIES HIVER', 'SORTIES VTT']

    for dir in dir_list_sorties:
        _ = create_folder(Path(ctg_path) / Path(year_folder) / Path(dir),
                         'CSV',
                          verbose = verbose)
        _ = create_folder(Path(ctg_path) / Path(year_folder) / Path(dir),
                          'EXCEL',
                          verbose = verbose)

    jours_sortie_club(year_folder,ctg_path)
    message = f"Architecture created for {year_folder} folder"
    return message

def jours_sortie_club(year,ctg_path):

    days_dict = {0: 'dimanche',
                 1: 'Lundi',
                 2: 'mardi',
                 3: 'mercredi',
                 4: 'jeudi',
                 5: 'vendredi',
                 6: 'samedi'}
    inv_days_dict = dict(zip(days_dict.values(),days_dict.keys()))
    month_dict = dict(zip(itertools.islice(itertools.cycle(l:=range(1,13)),2,2+len(l)),l))

    root = Path(ctg_path) / Path(str(year))

    for day in ['dimanche','samedi','jeudi']:
        path = ctg_path / Path(str(year)) / Path('SORTIES DU '+day.upper()) / Path('CSV')

        r = []
        rw = []
        for m in range(1,13):
            m_save=m
            y = int(year)%100
            c = int(int(year)/100)
            m = month_dict[m]
            if m>10 : y = y-1
            _,nb_days  = monthrange(y+200*c, m_save)
            for d in range (1,nb_days+1):
                if (d + int(2.6*m - 0.2) - 2*c + y + int(c/4) + int(y/4))%7 == inv_days_dict[day]:
                    r.append(f'{str(m_save).zfill(2)}_{str(d).zfill(2)} sortie du {day}.csv')
                    try:
                        num_week = datetime.date(year,m_save, d).isocalendar().week
                    except:
                        num_week = None

                    if num_week is not None:
                        rw.append((f'{str(m_save).zfill(2)}_{str(d).zfill(2)} sortie du {day}.csv' ,
                                        num_week))

                    file_name = f'{year}_{str(m_save).zfill(2)}_{str(d).zfill(2)} sortie du {day}.csv'
                    with open(os.path.join(path,file_name), 'w') as fp:
                        fp.write(f'{str(year)} SORTIES DU {day.upper()}')

def get_available_sorties(ctg_path,year):

    year_path = ctg_path / Path(str(year))
    dir_year_list = os.listdir(year_path)

    available_sorties_list = list(set(DIR_SORTIES_LIST).intersection(dir_year_list))

    return available_sorties_list


__all__ = ['AppMain']

# Standard library imports
import tkinter as tk
from datetime import datetime
from functools import partial
from pathlib import Path
from tkinter import filedialog
from tkinter import messagebox
from tkinter import font as tkFont

# 3rd party imports
from screeninfo import get_monitors

# Local imports
import ctg.ctggui.guiglobals as gg
import ctg.ctggui.guitools as guf
from ctg.ctggui.guitools import font_size
from ctg.ctggui.guitools import last_available_years
from ctg.ctggui.guitools import mm_to_px
from ctg.ctggui.guitools import place_after
from ctg.ctggui.guitools import str_size_mm
from ctg.ctggui.guitools import create_archi
from ctg.ctggui.guitools import window_properties
from ctg.ctggui.page_effectif import create_effectif_analysis
from ctg.ctggui.pagesorties import create_sorties_analysis
from ctg.ctggui.pagesynthese import create_synthese_analysis
from ctg.ctggui.pagetendance import create_tendance_analysis
from ctg.ctggui.pagesejour import create_sejour_analysis
from ctg.ctggui.pagedivers import create_divers_analysis
from ctg.ctggui.guitools import place_bellow
from ctg.ctggui.guitools import show_frame

class AppMain(tk.Tk):
    '''
    '''

    ############################### Class init - start ###############################
    def __init__(master):

        ctg_path_default = Path.home() /Path('CTG/SORTIES')

        # Setting the link with "tk.Tk"
        tk.Tk.__init__(master)


        # Identifying tk window of init class
        _ = get_monitors() # OBLIGATOIRE
        master.attributes("-topmost", True)

        # Setting the icon
        file = Path(__file__).parent.parent / Path('ctgfuncts') 
        file = file / Path('CTG_RefFiles') / Path( 'logoctg4.ico') 
        master.iconbitmap( )

        # Setting title window
        master.title(gg.APPLICATION_WINDOW_TITLE)

        # Setting window size depending on scale factor
        screen_width_px  = master.winfo_screenwidth()
        screen_height_px = master.winfo_screenheight()

        sizes_tuple = window_properties(screen_width_px, screen_height_px)

        win_width_px  = sizes_tuple[0]
        win_height_px = sizes_tuple[1]

        master.geometry(f"{win_width_px}x{win_height_px}")
        master.resizable(False, False)

        # Defining pages classes and pages list
        AppMain.pages = ( PageHelp,
                          PageDivers,
                          PageSejours,
                          PageTendance,
                          PageSynthese,
                          PageEffectif,
                          PageSorties,
                         )
        AppMain.pages_ordered_list = [x.__name__ for x in AppMain.pages][::-1]

        # Getting useful screen sizes and scale factors depending on displays properties
        AppMain.win_width_px  = sizes_tuple[0]
        AppMain.win_height_px = sizes_tuple[1]
        AppMain.width_sf_px   = sizes_tuple[2]
        AppMain.height_sf_px  = sizes_tuple[3]     # unused here
        AppMain.width_sf_mm   = sizes_tuple[4]
        AppMain.height_sf_mm  = sizes_tuple[5]
        AppMain.width_sf_min  = min(AppMain.width_sf_mm, AppMain.width_sf_px)


        ######################################## Title and copyright and application initialization
        PageTitle(master)
        AuthorCopyright(master)
        InitApp(master)
        SetLaunchButton(master, "CTG", ctg_path_default)


    ################################ Class init - end ################################

class InitApp(tk.Tk):

    def __init__(self,master):

        # Internal functions - start
        def _display_path(inst_bmf):
            """Shortening bmf path for easy display"""
            p = Path(inst_bmf)
            p_disp = ('/'.join(p.parts[0:2])) / Path("...") / ('/'.join(p.parts[-3:]))
            return p_disp

        def _get_file(institute_select):
            # Getting new working directory
            dialog_title = "Choisir un nouveau dossier de travail"
            bmf_str = tk.filedialog.askdirectory(title = dialog_title)
            if bmf_str == '':
                warning_title = "!!! Attention !!!"
                warning_text = "Chemin non renseigné."
                return messagebox.showwarning(warning_title, warning_text)

            # Updating bmf values using new working directory
            _set_bmf_widget_param(institute_select, bmf_str)
            _update_corpi(bmf_str)
            AppMain.institute = institute_select # FB
            AppMain.ctg_path = bmf_str # FB
            SetLaunchButton(master, institute_select, bmf_str)

        def _set_bmf_widget_param(institute_select, inst_bmf):
            # Setting bmf widgets parameters
            bmf_font = tkFont.Font(family = gg.FONT_NAME,
                                   size   = eff_bmf_font_size,
                                   weight = 'bold')
            bmf_label       = tk.Label(master,
                                       text = gg.TEXT_ROOT_CTG,
                                       font = bmf_font,)
            bmf_val         = tk.StringVar(master)
            bmf_val2        = tk.StringVar(master)
            bmf_entree      = tk.Entry(master, textvariable = bmf_val)
            bmf_entree2     = tk.Entry(master, textvariable = bmf_val2, width = eff_bmf_width)
            bmf_button_font = tkFont.Font(family = gg.FONT_NAME,
                                          size   = eff_button_font_size)
            bmf_button      = tk.Button(master,
                                        text = gg.TEXT_ROOT_CTG_CHANGE,
                                        font = bmf_button_font,
                                        command = lambda: _get_file(institute_select))
            # Placing bmf widgets
            bmf_label.place(x = eff_bmf_pos_x_px,
                            y = eff_bmf_pos_y_px,)

            text_width_mm, _ = str_size_mm(gg.TEXT_ROOT_CTG, bmf_font, gg.PPI)
            eff_path_pos_x_px = mm_to_px(text_width_mm + add_space_mm, gg.PPI)
            bmf_entree2.place(x = eff_path_pos_x_px,
                              y = eff_bmf_pos_y_px,)

            bmf_button.place(x = eff_path_pos_x_px,
                             y = eff_bmf_pos_y_px + eff_button_dy_px,)
            bmf_val.set(inst_bmf)
            bmf_val2.set((_display_path(inst_bmf)))

        def _create_corpus(inst_bmf):
            corpi_val = _set_corpi_widgets_param(inst_bmf)
            bmf_path = Path(inst_bmf)
            try:
                # Setting new corpus year folder name
                corpuses_list    = last_available_years(bmf_path, gg.FIRST_YEAR)
                last_corpus_year = corpuses_list[-1]
                current_year = datetime.now().year
                if int(last_corpus_year) == int(current_year):
                    message = ('Vous ne pouvez pas créer une nouvelle année '
                              f"tant que l'année en cours {current_year} ne sera pas "
                               "terminée.")
                    messagebox.showwarning("Warning",message)
                    corpi_val.set(str(corpuses_list))
                    return
                    
                new_corpus_year_folder = str(int(last_corpus_year) + 1)
                # Creating required folders for new corpus year
                message = create_archi(bmf_path, new_corpus_year_folder, verbose = False)
                print("\n",message)

                # Getting updated corpuses list
                corpuses_list = last_available_years(bmf_path, gg.FIRST_YEAR)

                # Setting corpi_val value to corpuses list
                corpi_val.set(str(corpuses_list))

            except FileNotFoundError:
                warning_title = "!!! ATTENTION : Dossier de travail inaccessible !!!"
                warning_text  = f"L'accès au dossier {bmf_path} est impossible."
                warning_text += f"\nChoisissez un autre dossier de travail."
                messagebox.showwarning(warning_title, warning_text)

                # Setting corpi_val value to empty string
                corpi_val.set("")

        def _set_corpi_widgets_param(inst_bmf):
            # Setting corpuses widgets parameters
            corpi_font   = tkFont.Font(family = gg.FONT_NAME,
                                       size   = eff_corpi_font_size,
                                       weight = 'bold')
            corpi_val    = tk.StringVar(master)
            corpi_entry  = tk.Entry(master, textvariable = corpi_val, width = eff_list_width)
            corpi_button_font = tkFont.Font(family = gg.FONT_NAME,
                                            size   = eff_button_font_size)
            corpi_label  = tk.Label(master,
                                    text = gg.TEXT_CORPUSES,
                                    font = corpi_font,)
            corpi_button = tk.Button(master,
                                     text = gg.TEXT_BOUTON_CREATION_CORPUS,
                                     font = corpi_button_font,
                                     command = lambda: _create_corpus(inst_bmf))
                        # Placing corpuses widgets
            corpi_label.place(x = eff_corpi_pos_x_px,
                              y = eff_corpi_pos_y_px,)

            text_width_mm, _ = str_size_mm(gg.TEXT_CORPUSES, corpi_font, gg.PPI)
            eff_list_pos_x_px = mm_to_px(text_width_mm + add_space_mm, gg.PPI)
            corpi_entry.place(x = eff_list_pos_x_px ,
                              y = eff_corpi_pos_y_px,)

            corpi_button.place(x = eff_list_pos_x_px,
                               y = eff_corpi_pos_y_px + eff_button_dy_px,)
            return corpi_val

        def _update_corpi(inst_bmf):
            corpi_val = _set_corpi_widgets_param(inst_bmf)
            bmf_path = Path(inst_bmf)
            try:
                # Getting updated corpuses list
                corpuses_list = last_available_years(bmf_path, gg.FIRST_YEAR)
                # Setting corpi_val value to corpuses list
                corpi_val.set(str(corpuses_list))

            except FileNotFoundError:
                warning_title = "!!! ATTENTION : Dossier de travail inaccessible !!!"
                warning_text  = f"L'accès au dossier {bmf_path} est impossible."
                warning_text += f"\nChoisissez un autre dossier de travail."
                messagebox.showwarning(warning_title, warning_text)

                # Setting corpi_val value to empty string
                corpi_val.set("")

        def _update_page():

            institute_select = 'CTG'
            inst_default_bmf = Path.home() / Path('CTG/SORTIES')

            # Managing working folder (bmf stands for "BiblioMeter_Files")
            _set_bmf_widget_param(institute_select, inst_default_bmf)

            # Managing corpus list
            corpi_val = _set_corpi_widgets_param(inst_default_bmf)
                # Setting and displating corpuses list initial values
            try:
                init_corpuses_list = last_available_years(inst_default_bmf, gg.FIRST_YEAR)
                corpi_val.set(str(init_corpuses_list))
            except FileNotFoundError:
                warning_title = "!!! ATTENTION : Dossier de travail inaccessible !!!"
                warning_text  = f"L'accès au dossier {inst_default_bmf} est impossible."
                warning_text += f"\nChoisissez un autre dossier de travail."
                messagebox.showwarning(warning_title, warning_text)
                # Setting corpuses list value to empty string
                corpi_val.set("")

            # Managing analysis launch button
            AppMain.institute = institute_select # FB
            AppMain.ctg_path = inst_default_bmf # FB
            SetLaunchButton(master, institute_select, inst_default_bmf)

        ############################## Internal functions - end ##############################

        ######################################## Main ########################################


        # Setting common parameters for widgets
        add_space_mm = gg.ADD_SPACE_MM
        eff_button_font_size = font_size(gg.REF_BUTTON_FONT_SIZE, AppMain.width_sf_min)

        ##################### Working-folder selection widgets parameters ######################
        # Setting effective value for bmf entry width
        eff_bmf_width = int(gg.REF_ENTRY_NB_CHAR * AppMain.width_sf_min)

        # Setting font size for bmf
        eff_bmf_font_size = font_size(gg.REF_SUB_TITLE_FONT_SIZE, AppMain.width_sf_min)

        # Setting reference positions in mm and effective ones in pixels for bmf
        eff_bmf_pos_x_px = mm_to_px(gg.REF_BMF_POS_X_MM * AppMain.height_sf_mm, gg.PPI)
        eff_bmf_pos_y_px = mm_to_px(gg.REF_BMF_POS_Y_MM * AppMain.height_sf_mm, gg.PPI)

        # Setting reference relative positions in mm and effective relative
        # Y positions in pixels for bmf change button
        eff_button_dy_px = mm_to_px(gg.REF_BUTTON_DY_MM * AppMain.height_sf_mm, gg.PPI)

        ##################### Corpuses-list-display widgets parameters ######################
        # Setting effective value for corpi setting width
        eff_list_width = int(gg.REF_ENTRY_NB_CHAR * AppMain.width_sf_min)

        # Setting font size for corpi
        eff_corpi_font_size  = font_size(gg.REF_SUB_TITLE_FONT_SIZE, AppMain.width_sf_min)

        # Setting reference positions in mm and effective ones in pixels for corpuses
        eff_corpi_pos_x_px = mm_to_px(gg.REF_CORPI_POS_X_MM * AppMain.height_sf_mm, gg.PPI)
        eff_corpi_pos_y_px = mm_to_px(gg.REF_CORPI_POS_Y_MM * AppMain.height_sf_mm, gg.PPI)

        _update_page()

class SetLaunchButton(tk.Tk):

    def __init__(self, master, institute, ctg_path):

        #tk.Frame.__init__(self) #FB

        # Setting font size for launch button
        eff_launch_font_size = font_size(gg.REF_LAUNCH_FONT_SIZE, AppMain.width_sf_min)

        # Setting x and y position in pixels for launch button
        launch_but_pos_x_px = AppMain.win_width_px  * 0.5
        launch_but_pos_y_px = AppMain.win_height_px * 0.8

        # Setting launch button
        launch_font = tkFont.Font(family = gg.FONT_NAME,
                                  size   = eff_launch_font_size,
                                  weight = 'bold')
        launch_button = tk.Button(master,
                                  text = gg.TEXT_BOUTON_LANCEMENT,
                                  font = launch_font,
                                  command = lambda: self._generate_pages(master,
                                                                         institute,
                                                                         ctg_path))
        # Placing launch button
        launch_button.place(x = launch_but_pos_x_px,
                            y = launch_but_pos_y_px,
                            anchor = "s")

    def _generate_pages(self, master, institute, ctg_path):

        '''Permet la génération des pages après spécification du chemin
        vers la zone de stockage.
        Vérifie qu'un chemin a été renseigné et continue le cas échant,
        sinon redemande de renseigner un chemin.
        '''

        if ctg_path == '':
            warning_title = "!!! Attention !!!"
            warning_text =  "Chemin non renseigné."
            warning_text += "\nL'application ne peut pas être lancée."
            warning_text += "\nVeuillez le définir."
            messagebox.showwarning(warning_title, warning_text)

        else:
            # Creating two frames in the tk window
            container_button = tk.Frame(master,
                                        height = gg.CONTAINER_BUTTON_HEIGHT_PX,
                                        bg = 'red')
            container_button.pack(side = "top",
                                  fill = "both",
                                  expand = False)

            container_frame = tk.Frame(master)
            container_frame.pack(side="top",
                                 fill="both",
                                 expand=True)
            container_frame.grid_rowconfigure(0,
                                              weight = 1)
            container_frame.grid_columnconfigure(0,
                                                 weight = 1)
            master.frames = {}
            for page in AppMain.pages:
                page_name = page.__name__
                print('----',page_name)
                frame = page(container_frame, master, container_button, institute, ctg_path)
                master.frames[page_name] = frame

                # put all of the pages in the same location;
                # the one on the top of the stacking order
                # will be the one that is visible.
                frame.grid(row = 0,
                           column = 0,
                           sticky = "nsew")


class PageTitle(tk.Tk):

    def __init__(self, parent):

        ####################### Title and copyright widgets parameters ########################
        # Setting font size for page title and copyright
        eff_page_title_font_size = guf.font_size(gg.REF_PAGE_TITLE_FONT_SIZE, AppMain.width_sf_min)

        # Setting reference Y position in mm and effective Y position in pixels for page label
        eff_page_title_pos_y_px = guf.mm_to_px(gg.REF_PAGE_TITLE_POS_Y_MM * AppMain.height_sf_mm,
                                               gg.PPI)

        # Setting x position in pixels for page title
        mid_page_pos_x_px = AppMain.win_width_px  * 0.5
        page_title = tk.Label(parent,
                              text = gg.TEXT_TITLE,
                              font = (gg.FONT_NAME, eff_page_title_font_size),
                              justify = "center")
        page_title.place(x = mid_page_pos_x_px,
                         y = eff_page_title_pos_y_px,
                         anchor = "center")

class AuthorCopyright(tk.Frame):

    def __init__(self, parent):

        # Setting font size for copyright
        ref_copyright_font_size = gg.REF_COPYRIGHT_FONT_SIZE
        eff_copyright_font_size = guf.font_size(ref_copyright_font_size, AppMain.width_sf_min)

        # Setting X and Y positions reference in mm for copyright
        ref_copyright_x_mm = gg.REF_COPYRIGHT_X_MM              #5
        eff_copyright_x_px = guf.mm_to_px(ref_copyright_x_mm * AppMain.width_sf_mm, gg.PPI)
        ref_copyright_y_mm = gg.REF_COPYRIGHT_Y_MM              #170
        eff_copyright_y_px = guf.mm_to_px(ref_copyright_y_mm * AppMain.height_sf_mm, gg.PPI)

        # Setting font size for version
        ref_version_font_size = gg.REF_VERSION_FONT_SIZE         #12
        eff_version_font_size = guf.font_size(ref_version_font_size,AppMain.width_sf_min)

        # Setting X and Y positions reference in mm for version
        ref_version_x_mm = gg.REF_VERSION_X_MM                   #185
        ref_version_y_mm = gg.REF_COPYRIGHT_Y_MM                 #170
        eff_version_y_px = guf.mm_to_px(ref_version_y_mm * AppMain.height_sf_mm, gg.PPI)
        eff_version_x_px = guf.mm_to_px(ref_version_x_mm * AppMain.width_sf_mm, gg.PPI)


        Auteurs_font_label = tkFont.Font(family = gg.FONT_NAME,
                                             size   = eff_copyright_font_size,)
        Auteurs_label = tk.Label(parent,
                                 text = gg.TEXT_COPYRIGHT,
                                 font = Auteurs_font_label,
                                 justify = "left")
        Auteurs_label.place(x = eff_copyright_x_px,
                            y = eff_copyright_y_px,
                            anchor = "sw")

        version_font_label = tkFont.Font(family = gg.FONT_NAME,
                                         size = eff_version_font_size,
                                         weight = 'bold')
        version_label = tk.Label(parent,
                                 text = gg.TEXT_VERSION,
                                 font = version_font_label,
                                 justify = "right")
        version_label.place(x = eff_version_x_px,
                            y = eff_version_y_px,
                            anchor = "sw")


class PageEffectif(tk.Frame):
    '''
    '''
    def __init__(self, container_frame, master, container_button, institute, ctg_path):

        super().__init__(container_frame)

        page_name  = self.__class__.__name__

        create_effectif_analysis(self, master, page_name, institute, ctg_path)

        setcontrollerbutton = SetControllerButton(master, container_button,page_name)

        settitleclass = SetTitltleClass(self,page_name,institute)

        quitapp = QuitApp(self, master, container_button)

class PageSejours(tk.Frame):
    '''
    '''
    def __init__(self, container_frame, master, container_button, institute, ctg_path):

        super().__init__(container_frame)

        page_name  = self.__class__.__name__

        create_sejour_analysis(self, master, page_name, institute, ctg_path)

        setcontrollerbutton = SetControllerButton(master, container_button,page_name)

        settitleclass = SetTitltleClass(self,page_name,institute)

        quitapp = QuitApp(self, master, container_button)

class PageSorties(tk.Frame):
    '''
    '''
    def __init__(self, container_frame, master, container_button, institute, ctg_path):

        super().__init__(container_frame)

        page_name  = self.__class__.__name__

        create_sorties_analysis(self, master, page_name, institute, ctg_path)

        setcontrollerbutton = SetControllerButton(master, container_button,page_name)

        settitleclass = SetTitltleClass(self,page_name,institute)

        quitapp = QuitApp(self, master, container_button)




class PageSynthese(tk.Frame):
    '''
    '''
    def __init__(self, container_frame, master, container_button, institute, ctg_path):

        super().__init__(container_frame)

        page_name  = self.__class__.__name__

        create_synthese_analysis(self, master, page_name, institute, ctg_path)

        setcontrollerbutton = SetControllerButton(master, container_button,page_name)

        settitleclass = SetTitltleClass(self,page_name,institute)

        quitapp = QuitApp(self, master, container_button)

class PageTendance(tk.Frame):
    '''
    '''
    def __init__(self, container_frame, master, container_button, institute, ctg_path):

        super().__init__(container_frame)

        page_name  = self.__class__.__name__

        create_tendance_analysis(self, master, page_name, institute, ctg_path)

        setcontrollerbutton = SetControllerButton(master, container_button,page_name)

        settitleclass = SetTitltleClass(self,page_name,institute)

        quitapp = QuitApp(self, master, container_button)

class PageDivers(tk.Frame):
    '''
    '''
    def __init__(self, container_frame, master, container_button, institute, ctg_path):

        super().__init__(container_frame)

        page_name  = self.__class__.__name__

        create_divers_analysis(self, master, page_name, institute, ctg_path)

        setcontrollerbutton = SetControllerButton(master, container_button,page_name)

        settitleclass = SetTitltleClass(self,page_name,institute)

        quitapp = QuitApp(self, master, container_button)

class PageHelp(tk.Frame):
    '''
    '''
    def __init__(self, container_frame, master, container_button, institute, ctg_path):

        if_analysis_x_pos_px     = mm_to_px(10 * AppMain.width_sf_mm,  gg.PPI)
        if_analysis_y_pos_px     = mm_to_px(10 * AppMain.height_sf_mm, gg.PPI)

        super().__init__(container_frame)

        page_name  = self.__class__.__name__

        # Text box and y slide definition
        help_box = tk.Text(self,
                           width=90,
                           height=30,
                           highlightthickness=1,
                           foreground="black",
                           insertbackground="black",
                           font=("Helvetica", 8))

        help_box.place(x = if_analysis_x_pos_px, y = if_analysis_y_pos_px)

        tex_scroll_y = tk.Scrollbar(self,orient=tk.VERTICAL,)
        tex_scroll_y.config(command=help_box.yview, )
        help_box["yscrollcommand"] = tex_scroll_y.set
        place_after(help_box, tex_scroll_y, dx = 5, dy = 0)

        # Write into the Text box
        #version_gui = ctg.ctggui.__version__
        #help_box.insert(tk.END,f'Version : {version_gui} \n\n')
        path_help = Path(__file__).parent.parent / Path('ctgfuncts') 
        path_help = path_help / Path('CTG_RefFiles') / Path( 'help.txt')
        with open(path_help,'r',encoding="utf-8")as file_text:
            help_content =  file_text.read()
        help_box.insert(tk.END,help_content)

        setcontrollerbutton = SetControllerButton(master, container_button,page_name)

        quitapp = QuitApp(self, master, container_button)

class SetControllerButton(tk.Tk):

    def __init__(self, master, container_button,page_name):

        page_num   = AppMain.pages_ordered_list.index(page_name)
        label_text = gg.PAGES_LABELS.setdefault(page_name, page_name)

        # Setting font size for page label and button
        eff_label_font_size  = font_size(gg.REF_LABEL_FONT_SIZE, AppMain.width_sf_min)
        eff_button_font_size = font_size(gg.REF_BUTTON_FONT_SIZE, AppMain.width_sf_min)

        button_font = tkFont.Font(family = gg.FONT_NAME,
                                  size   = eff_button_font_size)
        button = tk.Button(container_button,
                           text = label_text,
                           font = button_font,
                           command = lambda: show_frame(master, page_name))  ####FB
        button.grid(row = 0, column = page_num)


class SetTitltleClass(tk.Tk):

    def __init__(self, parent, page_name, institute):

        label_text = gg.PAGES_LABELS.setdefault(page_name, page_name)
        page_title = label_text + " du " + institute

        # Setting y_position in px for page label
        eff_label_pos_y_px = mm_to_px(gg.REF_LABEL_POS_Y_MM * AppMain.height_sf_mm, gg.PPI)

        eff_label_font_size  = font_size(gg.REF_LABEL_FONT_SIZE, AppMain.width_sf_min)
        # Setting x position in pixels for page label
        mid_page_pos_x_px = AppMain.win_width_px / 2


        label_font = tkFont.Font(family = gg.FONT_NAME,
                                 size   = eff_label_font_size)
        label = tk.Label(parent,
                         text = page_title,
                         font = label_font)
        label.place(x = mid_page_pos_x_px,
                    y = eff_label_pos_y_px,
                    anchor = "center")


class QuitApp(tk.Frame):

    def __init__(self, parent, master, container_button):

        super().__init__(parent)
        ################## Bouton pour sortir de la page
        eff_button_font_size = font_size(gg.REF_BUTTON_FONT_SIZE, AppMain.width_sf_min)
        eff_buttons_font_size    = font_size(gg.REF_ETAPE_FONT_SIZE-3, AppMain.width_sf_min)
        exit_button_x_pos_px     = mm_to_px(gg.REF_EXIT_BUT_POS_X_MM * AppMain.width_sf_mm,  gg.PPI)
        exit_button_y_pos_px     = mm_to_px(gg.REF_EXIT_BUT_POS_Y_MM * AppMain.height_sf_mm, gg.PPI)
        quit_font = tkFont.Font(family = gg.FONT_NAME,
                                size   = eff_buttons_font_size)

        quit_button = tk.Button(parent,
                                text = gg.TEXT_PAUSE,
                                font = quit_font,
                                command = lambda: self._launch_exit(master))
        quit_button.place(x = exit_button_x_pos_px,
                          y = exit_button_y_pos_px,
                          anchor = 'n')

    def _launch_exit(self,controller):

        message =  "Vous allez fermer CTG_Meter. "
        message += "\nRien ne sera perdu et vous pourrez reprendre le traitement plus tard."
        message += "\n\nSouhaitez-vous faire une pause dans le traitement ?"
        answer_1 = messagebox.askokcancel('Information', message)
        if answer_1:
            controller.destroy()


__all__ = ['built_lat_long',
           'day_of_the_date',
           'get_cout_total',
           'get_info_randos2df',
           'get_sejour_info',
           'normalize_tag',
           'parse_date',
           'read_sortie_csv',
           ]

# Standard library imports
import pathlib
import re
from collections import Counter
from collections import namedtuple
from datetime import datetime
from pathlib import Path

# 3rd party imports
import pandas as pd
from pandas.errors import EmptyDataError, ParserError

def parse_date(tag:str,year:int)->datetime:

    '''
    Convert a tag formatted as yyyy[_-]mm[_-]dd\s<text> or mm[_-]dd\s<text> in datetime format
    '''

    convert_to_date = lambda tag: datetime.strptime(tag,"%Y_%m_%d")

    tag = re.sub(r"-","_",tag)
    tag = tag+" "

    if re.findall(r'^\d{4}_\d{1,2}_\d{1,2}\s',tag):
        pattern = re.compile(r"(?P<year>\b\d{4}_)(?P<month>\d{1,2}_)(?P<day>\d{1,2})")
        match = pattern.search(tag)
        date = convert_to_date(match.group("year")+match.group("month")+match.group("day"))
    elif re.findall(r'^\d{1,2}_\d{1,2}\s',tag):
        pattern = re.compile(r"(?P<month>\d{1,2}_)(?P<day>\d{1,2})")
        match = pattern.search(tag)
        date = convert_to_date(str(year)+'_'+match.group("month")+match.group("day"))
    else:
        raise Exception(f'erreur in parse_date: unknown tag format {tag}')
    return date

def day_of_the_date(day:int,month:int,year:int)->str:

    '''Compute the day of the week of the date after
    "Elementary Number Theory David M. Burton Chap 6.4" [1]'''

    days_dict = {0: 'dimanche',
                 1: 'Lundi',
                 2: 'mardi',
                 3: 'mercredi',
                 4: 'jeudi',
                 5: 'vendredi',
                 6: 'samedi'}

    month_dict = {3: 1, 4: 2, 5: 3, 6: 4, 7: 5,
                  8: 6, 9: 7, 10: 8, 11: 9, 12:
                  10, 1: 11, 2: 12} # [1] p. 125

    y = year % 100
    c = int(year/100)
    m = month_dict[month]
    if m>10:
        y = y-1
    day_of_the_week = days_dict[(day + int(2.6*m - 0.2) -
                                2*c + y + int(c/4) +
                                int(y/4))%7] # [1] thm 6.12

    return day_of_the_week

def get_info_randos2df(ctg_path:pathlib.WindowsPath,year:str):

    '''Reads the Excel file info_randos of the year  `year`'''

    info_path = ctg_path / Path(str(year)) / Path('DATA') / Path('info_randos.xlsx')
    df = pd.read_excel(info_path)

    return df

def get_sejour_info(ctg_path:pathlib.WindowsPath,year:str)->tuple:

    '''get sejour information from the file `info_randos.xlsx` loacated in DATA
    of the year
    '''

    sejour_info = namedtuple('sejour_info', 'nbr_jours nbr_sejours histo')
    df =  get_info_randos2df(ctg_path,year)
    info_sejour = df.query('type=="sejour"')['nbr_jours'].tolist()

    c = Counter()
    c = Counter(info_sejour)


    sejour_info_tup = sejour_info( sum(info_sejour),len(info_sejour),c)

    return sejour_info_tup

def get_cout_total(year:str,type_sejour:str,
                   dg:pd.DataFrame,
                   ctg_path:pathlib.WindowsPath)->float:

    ''' Calcul du coût total des randonnées (type='randonnee") ou
    des séjours (type="sejour") pour l'année year
    '''

    file_info = Path(ctg_path) / Path(year) / Path('DATA') / Path('info_randos.xlsx')
    df_indo = pd.read_excel(file_info)
    cout_total = 0
    for evenement in dg.index:
        date_rando = f"{str(year)[2:4]}-{evenement[0:5].replace('_','-')}"
        cout_rando = df_indo.query('date==@date_rando and type==@type_sejour')['Cout'].tolist()[0]
        nbr_participants = dg[evenement]
        cout_total += cout_rando * nbr_participants

    return cout_total

def built_lat_long(df:pd.DataFrame)->pd.DataFrame:

    '''
    Add the two columns `long`and `lat` to the DataFrame df of the effectif.
    Return the DataFrame dg     with columns `Ville`, `long`, `lat`, `number`
    where number is the number of occurrence of ville in the DataFrame df
    '''
    path_villes_france = Path('ctgfuncts/CTG_RefFiles/villes_france_premium.csv')
    path_villes_de_france = Path(__file__).parent.parent / path_villes_france

    def normalize_ville(x):

        '''normalize ville name to stick with the convention of ville_de_france file and to
        avoid ambiguity
        '''
        dic_ville = {'SAINT-HILAIRE-DU-TOUVET':"SAINT-HILAIRE-38",
                     'SAINT-HILAIRE':"SAINT-HILAIRE-38",
                     'LAVAL-EN-BELLEDONNE':'LAVAL-38',
                     'LAVAL':"LAVAL-38",
                     'CRETS-EN-BELLEDONNE':"SAINT-PIERRE-D'ALLEVARD"}
        if x in dic_ville.keys():
            return dic_ville[x]
        else:
            return x

    df_villes = pd.read_csv(path_villes_de_france,header=None,usecols=[3,19,20])
    dic_long = dict(zip(df_villes[3] , df_villes[19]))
    dic_lat = dict(zip(df_villes[3] , df_villes[20]))

    df['Ville'] = df['Ville'].str.replace(' ','-')
    df['Ville'] = df['Ville'].str.replace('ST-','SAINT-')
    df['Ville'] = df['Ville'].str.replace(r'\-D\-+',"-D'",regex=True)
    df['Ville'] = df['Ville'].str.replace('^LA-',"LA ",regex=True)
    df['Ville'] = df['Ville'].str.replace('^LE-',"LE ",regex=True)

    df['Ville'] = df['Ville'].apply(normalize_ville)


    df['long'] = df['Ville'].map(dic_long)
    df['lat'] = df['Ville'].map(dic_lat)
    list_villes = df['Ville'].tolist()
    Counter(list_villes)
    dg = df.groupby(['Ville']).count()['N° Licencié']

    dh = pd.DataFrame.from_dict({'Ville':dg.index,
                                'long':dg.index.map(dic_long),
                                'lat':dg.index.map(dic_lat),
                                'number':dg.tolist()})
    return dh

def read_sortie_csv(file:pathlib.WindowsPath):

    '''
    '''
    try:
        err = 0
        df = pd.read_csv(file,skiprows=1,header=None)
    except EmptyDataError:
        df = None
    except ParserError :
        print(f'WARNING : The csv file {file} is hill configurated')
        df = None

    return df
    
def normalize_tag(tag:str,year:str)->str:

    '''
    Convert a tag formatted as yyyy[_-]mm[_-]dd\s<text> or mm[_-]dd\s<text> in 
    yy-mm-dd where year is used if missing in the tag
    '''
    
    tag = re.sub(r"_","-",tag)
    tag = tag+" "
   
    if re.findall(r'^\d{4}-\d{1,2}-\d{1,2}\s',tag):
        return tag[2:10]
        
    elif re.findall(r'^\d{1,2}-\d{1,2}\s',tag):
        return f'{year[2:4]}-{tag[0:5]}'
    else:
        raise Exception(f'erreur in normaize_tag: unknown tag format {tag}')



__all__ = ['create_divers_analysis']

# Standard library imports
import os
import shutil
from pathlib import Path
import tkinter as tk
from tkinter import font as tkFont
from tkinter import filedialog
from tkinter import messagebox
from pathlib import Path

# 3rd party imports
import pandas as pd

# Local imports
import ctg.ctggui.guiglobals as gg
from ctg.ctggui.guitools import encadre_RL
from ctg.ctggui.guitools import font_size
from ctg.ctggui.guitools import mm_to_px
from ctg.ctggui.guitools import place_after
from ctg.ctggui.guitools import place_bellow
from ctg.ctggui.guitools import last_available_years
from ctg.ctgfuncts.ctg_plot import plot_ctg
from ctg.ctgfuncts.ctg_synthese import nbr_sejours_adherent

def create_divers_analysis(self, master, page_name, institute, ctg_path):

    """
    Description : function working as a bridge between the BiblioMeter
    App and the functionalities needed for the use of the app

    Uses the following globals :
    - DIC_OUT_PARSING
    - FOLDER_NAMES

    Args: takes only self and ctg_path as arguments.
    self is the instense in which PageThree will be created
    ctg_path is a type Path, and is the path to where the folders
    organised in a very specific way are stored

    Returns : nothing, it create the page in self
    """

    # Internal functions
    def _launch_geo_analysis():

        # Getting year selection
        year_select =  variable_years.get()
        kol = plot_ctg(ctg_path,year_select)

        path_html = ctg_path / Path(str(year_select)) / Path('STATISTIQUES') / Path('HTML') 
        path_html = path_html / Path("ctg.html")
        kol.save(path_html)

        info_title = "- Information -"
        info_text  = ("L'analyse de la localisation géographique des membres a été effectuée"
                      f"pour l'année {year_select} "
                      f"\n\nLe fichier .html obtenu : 'ctg.html' "
                      "a été créé dans le dossier :"
                      f"\n\n{ctg_path}\{str(year_select)}\STATISTIQUES\HTML.")

        messagebox.showinfo(info_title, info_text)

    def _launch_participation_sejour(ctg_path):

        # Getting year selection
        year_select =  variable_years.get()

        nbr_sejours_adherent(year_select,ctg_path)

    from ctg.ctggui.pageclasses import AppMain

    # Setting effective font sizes and positions (numbers are reference values)
    eff_etape_font_size      = font_size(gg.REF_ETAPE_FONT_SIZE,   AppMain.width_sf_min)
    eff_launch_font_size     = font_size(gg.REF_ETAPE_FONT_SIZE-1, AppMain.width_sf_min)
    eff_help_font_size       = font_size(gg.REF_ETAPE_FONT_SIZE-2, AppMain.width_sf_min)
    eff_select_font_size     = font_size(gg.REF_ETAPE_FONT_SIZE, AppMain.width_sf_min)
    eff_buttons_font_size    = font_size(gg.REF_ETAPE_FONT_SIZE-3, AppMain.width_sf_min)

    synthese_analysis_x_pos_px     = mm_to_px(10 * AppMain.width_sf_mm,  gg.PPI)
    synthese_analysis_y_pos_px     = mm_to_px(40 * AppMain.height_sf_mm, gg.PPI)
    year_analysis_label_dx_px  = mm_to_px( 0 * AppMain.width_sf_mm,  gg.PPI)
    year_analysis_label_dy_px  = mm_to_px(15 * AppMain.height_sf_mm, gg.PPI)
    launch_dx_px             = mm_to_px( 0 * AppMain.width_sf_mm,  gg.PPI)
    launch_dy_px             = mm_to_px( 5 * AppMain.height_sf_mm, gg.PPI)

    year_button_x_pos        = mm_to_px(gg.REF_YEAR_BUT_POS_X_MM * AppMain.width_sf_mm,  gg.PPI)
    year_button_y_pos        = mm_to_px(gg.REF_YEAR_BUT_POS_Y_MM * AppMain.height_sf_mm, gg.PPI)
    dy_year                  = -6
    ds_year                  = 5

    # Setting common attributs
    etape_label_format = 'left'
    etape_underline    = -1

    ### Choix de l'année
    list_years = last_available_years(ctg_path,year_number=2020)[1:]
    default_year = list_years[-1]
    variable_years = tk.StringVar(self)
    variable_years.set(default_year)

        # Création de l'option button des années
    self.font_OptionButton_years = tkFont.Font(family = gg.FONT_NAME,
                                               size = eff_buttons_font_size)
    self.OptionButton_years = tk.OptionMenu(self,
                                            variable_years,
                                            *list_years)
    self.OptionButton_years.config(font = self.font_OptionButton_years)

        # Création du label
    self.font_Label_years = tkFont.Font(family = gg.FONT_NAME,
                                        size = eff_select_font_size,
                                        weight = 'bold')
    self.Label_years = tk.Label(self,
                                text = gg.TEXT_GEO_PI,
                                font = self.font_Label_years)
    self.Label_years.place(x = year_button_x_pos, y = year_button_y_pos)

    place_after(self.Label_years, self.OptionButton_years, dy = dy_year)

    ################## Synthèse des activité

    ### Titre
    synthese_analysis_font = tkFont.Font(family = gg.FONT_NAME,
                                         size = eff_etape_font_size,
                                         weight = 'bold')
    synthese_analysis_label = tk.Label(self,
                                       text = gg.TEXT_GEO,
                                       justify = etape_label_format,
                                       font = synthese_analysis_font,
                                       underline = etape_underline)

    synthese_analysis_label.place(x = synthese_analysis_x_pos_px,
                                  y = synthese_analysis_y_pos_px)

    ### Explication
    help_label_font = tkFont.Font(family = gg.FONT_NAME,
                                  size = eff_help_font_size)
    help_label = tk.Label(self,
                          text = gg.HELP_GEO,
                          justify = "left",
                          font = help_label_font)
    place_bellow(synthese_analysis_label,
                 help_label)

    ### Bouton pour lancer l'analyse des IFs
    synthese_analysis_launch_font = tkFont.Font(family = gg.FONT_NAME,
                                                size = eff_launch_font_size)
    synthese_analysis_launch_button = tk.Button(self,
                                                text = gg.BUTT_GEO,
                                                font = synthese_analysis_launch_font,
                                                command = lambda: _launch_geo_analysis())
    place_bellow(help_label,
                 synthese_analysis_launch_button,
                 dx = launch_dx_px,
                 dy = launch_dy_px)

    ################## Analyse des mots clefs

    ### Titre
    member_analysis_label_font = tkFont.Font(family = gg.FONT_NAME,
                                             size = eff_etape_font_size,
                                             weight = 'bold')
    member_analysis_label = tk.Label(self,
                                     text = gg.TEXT_NBR_SEJOURS,
                                     justify = "left",
                                     font = member_analysis_label_font)
    place_bellow(synthese_analysis_launch_button,
                 member_analysis_label,
                 dx = year_analysis_label_dx_px,
                 dy = year_analysis_label_dy_px)

    ### Explication de l'étape
    help_label_font = tkFont.Font(family = gg.FONT_NAME,
                                  size = eff_help_font_size)
    help_label = tk.Label(self,
                          text = gg.HELP_NBR_SEJOURS,
                          justify = "left",
                          font = help_label_font)
    place_bellow(member_analysis_label,
                 help_label)

    ### Bouton pour lancer l'analyse des mots clefs
    member_analysis_launch_font = tkFont.Font(family = gg.FONT_NAME,
                                                size = eff_launch_font_size)
    member_analysis_button = tk.Button(self,
                                         text = gg.BUTT_NBR_SEJOURS,
                                         font = member_analysis_launch_font,
                                         command = lambda: _launch_participation_sejour(ctg_path))
    place_bellow(help_label,
                 member_analysis_button,
                 dx = launch_dx_px,
                 dy = launch_dy_px)

################## Analyse de l'évolution de la moyenne d'âge

    ### Titre
    #age_analysis_label_font = tkFont.Font(family = gg.FONT_NAME,
    #                                      size = eff_etape_font_size,
    #                                      weight = 'bold')
    #age_analysis_label = tk.Label(self,
    #                              text = gg.TEXT_AGE_ANALYSIS,
    #                              justify = "left",
    #                              font = age_analysis_label_font)
    #place_bellow(member_analysis_button,
    #             age_analysis_label,
    #             dx = year_analysis_label_dx_px,
    #             dy = year_analysis_label_dy_px)
    #
    #### Explication de l'étape
    #help_label_font = tkFont.Font(family = gg.FONT_NAME,
    #                           size = eff_help_font_size)
    #help_label_age = tk.Label(self,
    #                          text = gg.HELP_AGE_ANALYSIS,
    #                          justify = "left",
    #                          font = help_label_font)
    #place_bellow(age_analysis_label,
    #             help_label_age)
    #
    #### Bouton pour lancer l'analyse des mots clefs
    #age_analysis_launch_font = tkFont.Font(family = gg.FONT_NAME,
    #                                     size = eff_launch_font_size)
    #age_analysis_button = tk.Button(self,
    #                                text = gg.BUTT_AGE_ANALYSIS,
    #                                font = age_analysis_launch_font,
    #                                command = lambda: _launch_age_analysis(ctg_path))
    #place_bellow(help_label_age,
    #             age_analysis_button,
    #             dx = launch_dx_px,
    #             dy = launch_dy_px)